/// <reference types="react" />
import * as React from 'react';
import { GraphQLEditor } from './Playground/GraphQLEditor';
import { PermissionSession, ServiceInformation, Session } from '../types';
import PlaygroundStorage from './PlaygroundStorage';
import { DocsState } from '../reducers/graphiql-docs';
export declare type Theme = 'dark' | 'light';
export declare type Viewer = 'ADMIN' | 'EVERYONE' | 'USER';
export interface Response {
    date: string;
    time: Date;
}
export interface Props {
    endpoint: string;
    subscriptionsEndpoint?: string;
    projectId?: string;
    adminAuthToken?: string;
    onSuccess?: (graphQLParams: any, response: any) => void;
    isEndpoint?: boolean;
    onboardingStep?: string;
    tether?: any;
    nextStep?: () => void;
    isApp?: boolean;
    setStacks?: (stack: any[]) => void;
    onChangeEndpoint?: (endpoint: string) => void;
    share: (state: any) => void;
    shareUrl?: string;
    session?: any;
    onChangeSubscriptionsEndpoint?: (endpoint: string) => void;
    getRef?: (ref: Playground) => void;
}
export interface State {
    schema: any;
    sessions: Session[];
    selectedSessionIndex: number;
    schemaCache: any;
    permissionSchema?: any;
    historyOpen: boolean;
    history: Session[];
    adminAuthToken?: string;
    response?: Response;
    selectUserOpen: boolean;
    userFields: string[];
    selectUserSessionId?: string;
    codeGenerationPopupOpen: boolean;
    disableQueryHeader: boolean;
    theme: Theme;
    autoReloadSchema: boolean;
    useVim: boolean;
    userModelName: string;
    shareAllTabs: boolean;
    shareHttpHeaders: boolean;
    shareHistory: boolean;
    changed: boolean;
    serviceInformation?: ServiceInformation;
}
export interface CursorPosition {
    line: number;
    ch: number;
}
export { GraphQLEditor };
export declare class Playground extends React.PureComponent<Props & DocsState, State> {
    storage: PlaygroundStorage;
    wsConnections: {
        [sessionId: string]: any;
    };
    observers: {
        [sessionId: string]: any;
    };
    graphiqlComponents: any[];
    private initialIndex;
    private schemaReloadInterval;
    private rawSchemaCache;
    private updateQueryTypes;
    private handleQueryChange;
    constructor(props: any);
    componentWillMount(): void;
    componentDidMount(): void;
    componentDidUpdate(prevProps: Props): void;
    fetchSchemas: () => any;
    componentWillUnmount(): void;
    setWS: (session: Session) => void;
    initWebsockets(): void;
    setCursor(position: CursorPosition): void;
    renewStack(schema: any): void;
    getNewStack(root: any, schema: any, stack: any): any;
    extractUserField(simpleSchema: any, userModelName?: string): any[];
    fetchPermissionSchema(): void;
    fetchServiceInformation(): void;
    getModelNames(): string[];
    extractServiceId(): string;
    getPermissionEndpoint(): string;
    fetchSchema(endpointUrl: string, headers?: any): any;
    render(): JSX.Element;
    renderUserPopup(): JSX.Element;
    handleUserModelChange: (userModelName: string) => void;
    renderCodeGenerationPopup(): JSX.Element;
    getDefaultPermissionQuery(session: PermissionSession): string;
    newPermissionTab: (permissionSession: PermissionSession, name: string, absolutePath: string, query: string) => void;
    handleNewPermissionTab: (permissionSession: PermissionSession) => void;
    setRef: (index: number, ref: any) => void;
    toggleSchemaReload: () => void;
    newSession: (name?: string | undefined) => void;
    closeTab: () => boolean;
    nextTab: () => void;
    prevTab: () => void;
    handleNewSession: (newIndexZero?: boolean) => void;
    setValueInSession(sessionId: string, key: string, value: any, cb?: () => void): void;
    private autofillMutation;
    private toggleTheme;
    private handleClickCodeGeneration;
    private handleCloseCodeGeneration;
    private handleUserSelection;
    private resetSubscriptions();
    private resetSubscription(session);
    private getUrlSession(sessions);
    private handleCreateSession;
    private handleItemStarToggled;
    private handleCloseSession;
    private handleOpenHistory;
    private handleCloseHistory;
    private handleSelectSession;
    private initSessions;
    private initOnboardingSessions();
    private saveSessions;
    private saveHistory;
    private handleNewSessionWithoutNewIndexZero;
    private createSession;
    private createSessionFromQuery;
    private handleChangeHeaders;
    private handleViewerChange;
    private handleCloseSelectUser;
    private handleVariableChange;
    private handleOperationNameChange;
    private setValueInHistory(sessionId, key, value);
    private isGraphcoolUrl(endpoint);
    private getSimpleEndpoint();
    private getSystemEndpoint();
    readonly httpApiPrefix: string;
    readonly wsApiPrefix: string;
    private getWSEndpoint();
    private addToHistory(session);
    private historyIncludes(session);
    private cancelSubscription;
    private permissionFetcher;
    private fetcher;
    private toggleUseVim;
    private toggleShareAllTabs;
    private toggleShareHTTPHeaders;
    private toggleShareHistory;
    private share;
}
declare const _default: React.ComponentClass<Props>;
export default _default;
