"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _style = require("styled-jsx/style");

var _style2 = _interopRequireDefault(_style);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var __extends = undefined && undefined.__extends || function () {
    var extendStatics = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function (d, b) {
        d.__proto__ = b;
    } || function (d, b) {
        for (var p in b) {
            if (b.hasOwnProperty(p)) d[p] = b[p];
        }
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
}();
var __assign = undefined && undefined.__assign || Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) {
            if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var GraphQLEditor_1 = require("./Playground/GraphQLEditor");
exports.GraphQLEditor = GraphQLEditor_1.GraphQLEditor;
var fetch = require("isomorphic-fetch");
var graphql_1 = require("graphql");
var TabBar_1 = require("./Playground/TabBar");
var constants_1 = require("../constants");
var cuid = require("cuid");
var Immutable = require("seamless-immutable");
var ThemeProvider_1 = require("./Theme/ThemeProvider");
var PlaygroundStorage_1 = require("./PlaygroundStorage");
var getQueryTypes_1 = require("./Playground/util/getQueryTypes");
var debounce_1 = require("graphiql/dist/utility/debounce");
var Observable_1 = require("rxjs/Observable");
var subscriptions_transport_ws_1 = require("subscriptions-transport-ws");
var isQuerySubscription_1 = require("./Playground/util/isQuerySubscription");
var HistoryPopup_1 = require("./HistoryPopup");
var cx = require("classnames");
var SelectUserPopup_1 = require("./SelectUserPopup");
var calculate_size_1 = require("calculate-size");
var CodeGenerationPopup_1 = require("./CodeGenerationPopup/CodeGenerationPopup");
var GraphDocs_1 = require("./Playground/DocExplorer/GraphDocs");
var data_1 = require("../data");
var Settings_1 = require("./Settings");
var react_redux_1 = require("react-redux");
var GraphQLEditorSession_1 = require("./Playground/GraphQLEditorSession");
var utils_1 = require("./Playground/DocExplorer/utils");
var graphiql_docs_1 = require("../actions/graphiql-docs");
var lodash_1 = require("lodash");
var Share_1 = require("./Share");
var NewPermissionTab_1 = require("./Permissions/NewPermissionTab");
var constants_2 = require("./constants");
var Playground = /** @class */function (_super) {
    __extends(Playground, _super);
    function Playground(props) {
        var _this = _super.call(this, props) || this;
        _this.wsConnections = {};
        _this.observers = {};
        _this.graphiqlComponents = [];
        _this.initialIndex = -1;
        _this.rawSchemaCache = null;
        _this.updateQueryTypes = debounce_1.default(150, function (sessionId, query) {
            var queryTypes = getQueryTypes_1.default(query);
            _this.setValueInSession(sessionId, 'queryTypes', queryTypes);
        });
        _this.handleQueryChange = debounce_1.default(300, function (sessionId, query) {
            _this.setValueInSession(sessionId, 'query', query);
            _this.setValueInSession(sessionId, 'hasChanged', true);
            _this.updateQueryTypes(sessionId, query);
            if (_this.props.onboardingStep === 'STEP3_UNCOMMENT_DESCRIPTION' && _this.state.selectedSessionIndex === 0) {
                var trimmedQuery = query.replace(/\s/g, '').replace(',', '');
                if (trimmedQuery === data_1.onboardingQuery1Check && typeof _this.props.nextStep === 'function') {
                    _this.props.nextStep();
                }
            }
            if ((_this.props.onboardingStep === 'STEP3_ENTER_MUTATION1_VALUES' || _this.props.onboardingStep === 'STEP3_ENTER_MUTATION2_VALUE') && _this.state.selectedSessionIndex === 1) {
                var trimmedTemplate = data_1.onboardingEmptyMutation.replace(/\s/g, '');
                var trimmedQuery = query.replace(/\s/g, '').replace(',', '');
                if (trimmedQuery !== trimmedTemplate && typeof _this.props.nextStep === 'function') {
                    _this.props.nextStep();
                }
            }
        });
        _this.fetchSchemas = function () {
            var additionalHeaders = {};
            var headers = _this.state.sessions[_this.state.selectedSessionIndex].headers;
            if (headers) {
                headers.forEach(function (header) {
                    return additionalHeaders[header.name] = header.value;
                });
            }
            return _this.fetchSchema(_this.getSimpleEndpoint(), additionalHeaders).then(function (simpleSchemaData) {
                if (!simpleSchemaData || simpleSchemaData.error) {
                    _this.setState({
                        response: {
                            date: simpleSchemaData.error,
                            time: new Date()
                        }
                    });
                    return;
                }
                if (lodash_1.isEqual(_this.rawSchemaCache, simpleSchemaData.data)) {
                    return;
                }
                _this.rawSchemaCache = simpleSchemaData.data;
                if (!simpleSchemaData.data) {
                    return;
                }
                var simpleSchema = graphql_1.buildClientSchema(simpleSchemaData.data);
                var userFields = _this.extractUserField(simpleSchema);
                _this.renewStack(simpleSchema);
                _this.setState({
                    schemaCache: simpleSchema,
                    userFields: userFields
                });
            });
        };
        _this.setWS = function (session) {
            var connectionParams = {};
            if (session.headers) {
                session.headers.forEach(function (header) {
                    connectionParams[header.name] = header.value;
                });
            }
            if (session.selectedViewer === 'ADMIN' && _this.state.adminAuthToken) {
                connectionParams.Authorization = "Bearer " + _this.state.adminAuthToken;
            } else if (session.selectedViewer === 'USER' && session.selectedUserToken) {
                connectionParams.Authorization = "Bearer " + session.selectedUserToken;
            }
            if (_this.wsConnections[session.id]) {
                _this.wsConnections[session.id].unsubscribeAll();
            }
            _this.wsConnections[session.id] = new subscriptions_transport_ws_1.SubscriptionClient(_this.getWSEndpoint(), {
                timeout: 20000,
                connectionParams: connectionParams
            });
        };
        _this.handleUserModelChange = function (userModelName) {
            var userFields = _this.extractUserField(_this.state.schemaCache, userModelName);
            _this.setState({
                userModelName: userModelName,
                userFields: userFields
            });
        };
        _this.newPermissionTab = function (permissionSession, name, absolutePath, query) {
            var newSession = Immutable({
                id: cuid(),
                selectedViewer: 'ADMIN',
                name: name,
                query: query,
                variables: '',
                result: '',
                operationName: undefined,
                hasChanged: true,
                hasQuery: false,
                permission: permissionSession,
                queryTypes: getQueryTypes_1.default(query),
                starred: false,
                absolutePath: absolutePath
            });
            _this.setState(function (state) {
                return __assign({}, state, { sessions: state.sessions.concat(newSession), selectedSessionIndex: state.sessions.length, changed: true });
            });
        };
        _this.handleNewPermissionTab = function (permissionSession) {
            var query = _this.getDefaultPermissionQuery(permissionSession);
            var newSession = Immutable({
                id: cuid(),
                selectedViewer: 'ADMIN',
                query: query,
                variables: '',
                result: '',
                operationName: undefined,
                hasQuery: false,
                permission: permissionSession,
                queryTypes: getQueryTypes_1.default(query),
                starred: false
            });
            _this.setState(function (state) {
                return __assign({}, state, { sessions: state.sessions.concat(newSession), selectedSessionIndex: state.sessions.length, changed: true });
            });
        };
        _this.setRef = function (index, ref) {
            _this.graphiqlComponents[index] = ref;
        };
        _this.toggleSchemaReload = function () {
            _this.setState(function (state) {
                return __assign({}, state, { autoReloadSchema: !state.autoReloadSchema });
            });
        };
        _this.newSession = function (name) {
            var session = _this.createSession();
            if (name) {
                session = Immutable.set(session, 'name', name);
            }
            _this.setState(function (state) {
                return __assign({}, state, { sessions: state.sessions.concat(session), selectedSessionIndex: state.sessions.length, changed: true });
            });
        };
        _this.closeTab = function () {
            var _a = _this.state,
                sessions = _a.sessions,
                selectedSessionIndex = _a.selectedSessionIndex;
            if (sessions.length > 1) {
                _this.handleCloseSession(sessions[selectedSessionIndex]);
                return true;
            }
            return false;
        };
        _this.nextTab = function () {
            var _a = _this.state,
                sessions = _a.sessions,
                selectedSessionIndex = _a.selectedSessionIndex;
            var numberOfSessions = sessions.length;
            if (numberOfSessions > 1) {
                _this.setState(function (state) {
                    return __assign({}, state, { selectedSessionIndex: selectedSessionIndex < numberOfSessions - 1 ? selectedSessionIndex + 1 : 0 });
                });
            }
        };
        _this.prevTab = function () {
            var _a = _this.state,
                sessions = _a.sessions,
                selectedSessionIndex = _a.selectedSessionIndex;
            var numberOfSessions = sessions.length;
            if (numberOfSessions > 1) {
                _this.setState(function (state) {
                    return __assign({}, state, { selectedSessionIndex: selectedSessionIndex > 0 ? selectedSessionIndex - 1 : numberOfSessions - 1 });
                });
            }
        };
        _this.handleNewSession = function (newIndexZero) {
            if (newIndexZero === void 0) {
                newIndexZero = false;
            }
            var session = _this.createSession();
            if (_this.props.onboardingStep === 'STEP3_CREATE_MUTATION_TAB') {
                session = Immutable.set(session, 'query', data_1.onboardingEmptyMutation);
                setTimeout(function () {
                    _this.setCursor({
                        line: 2,
                        ch: 15
                    });
                }, 5);
                if (typeof _this.props.nextStep === 'function') {
                    _this.props.nextStep();
                }
            }
            if (session.query === constants_1.defaultQuery) {
                setTimeout(function () {
                    _this.setCursor({
                        line: 1,
                        ch: 0
                    });
                }, 5);
            }
            _this.setState(function (state) {
                return __assign({}, state, { sessions: state.sessions.concat(session), selectedSessionIndex: newIndexZero ? 0 : state.sessions.length, changed: true });
            });
        };
        _this.autofillMutation = function () {
            var sessionId = _this.state.sessions[_this.state.selectedSessionIndex].id;
            if (_this.props.onboardingStep === 'STEP3_ENTER_MUTATION1_VALUES') {
                _this.setValueInSession(sessionId, 'query', data_1.onboardingFilledMutation1);
            } else if (_this.props.onboardingStep === 'STEP3_ENTER_MUTATION2_VALUE') {
                _this.setValueInSession(sessionId, 'query', data_1.onboardingFilledMutation2);
            }
            if (typeof _this.props.nextStep === 'function') {
                _this.props.nextStep();
            }
        };
        _this.toggleTheme = function () {
            _this.setState(function (state) {
                var theme = state.theme === 'dark' ? 'light' : 'dark';
                localStorage.setItem('theme', theme);
                return __assign({}, state, { theme: theme });
            });
        };
        _this.handleClickCodeGeneration = function () {
            _this.setState({
                codeGenerationPopupOpen: true
            });
        };
        _this.handleCloseCodeGeneration = function () {
            _this.setState({ codeGenerationPopupOpen: false });
        };
        _this.handleUserSelection = function (user) {
            var systemApi = _this.getSystemEndpoint();
            var query = "\n      mutation {\n        generateNodeToken(input: {\n          rootToken: \"" + _this.props.adminAuthToken + "\"\n          serviceId: \"" + _this.extractServiceId() + "\"\n          nodeId: \"" + user.id + "\"\n          modelName: \"" + _this.state.userModelName + "\"\n          clientMutationId: \"\"\n        }) {\n          clientMutationId\n          token\n        }\n      }\n    ";
            fetch(systemApi, {
                method: 'post',
                body: JSON.stringify({ query: query }),
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: "Bearer " + _this.state.adminAuthToken
                }
            }).then(function (res) {
                return res.json();
            }).then(function (res) {
                var token = res.data.generateNodeToken.token;
                if (token && _this.state.selectUserSessionId) {
                    var session = _this.state.sessions[_this.state.selectedSessionIndex];
                    var newHeaders = session.headers ? session.headers.filter(function (h) {
                        return h.name !== 'Authorization';
                    }) : [];
                    newHeaders = newHeaders.concat({
                        name: 'Authorization',
                        value: "Bearer " + token
                    });
                    _this.setValueInSession(_this.state.selectUserSessionId, 'headers', newHeaders, function () {
                        var concreteSession = _this.state.sessions[_this.state.selectedSessionIndex];
                        _this.resetSubscription(concreteSession);
                    });
                }
            });
        };
        _this.handleCreateSession = function (session) {
            var newSession = _this.createSession(session);
            _this.setState(function (state) {
                return __assign({}, state, { sessions: state.sessions.concat(newSession), selectedSessionIndex: state.sessions.length });
            });
        };
        _this.handleItemStarToggled = function (item) {
            _this.setValueInHistory(item.id, 'starred', !item.starred);
        };
        _this.handleCloseSession = function (session) {
            if (_this.state.sessions.length === 1) {
                _this.handleNewSession(true);
            }
            _this.setState(function (state) {
                var i = state.sessions.findIndex(function (s) {
                    return s.id === session.id;
                });
                var nextSelectedSession = state.selectedSessionIndex;
                if (nextSelectedSession > state.sessions.length - 2) {
                    // if it's not the last session
                    if (state.sessions.length > 1) {
                        nextSelectedSession--;
                    }
                }
                return __assign({}, state, { sessions: state.sessions.slice(0, i).concat(state.sessions.slice(i + 1, state.sessions.length)), selectedSessionIndex: nextSelectedSession, changed: true });
            });
            _this.storage.removeSession(session);
        };
        _this.handleOpenHistory = function () {
            _this.setState({ historyOpen: true });
        };
        _this.handleCloseHistory = function () {
            _this.setState({ historyOpen: false });
        };
        _this.handleSelectSession = function (session) {
            _this.setState(function (state) {
                var i = state.sessions.findIndex(function (s) {
                    return s.id === session.id;
                });
                if (_this.props.onboardingStep === 'STEP3_SELECT_QUERY_TAB' && i === 0 && typeof _this.props.nextStep === 'function') {
                    _this.props.nextStep();
                }
                return __assign({}, state, { selectedSessionIndex: i });
            });
        };
        _this.initSessions = function () {
            if (['STEP3_UNCOMMENT_DESCRIPTION', 'STEP3_OPEN_PLAYGROUND'].indexOf(_this.props.onboardingStep || '') > -1) {
                return _this.initOnboardingSessions();
            }
            // defaulting to admin for deserialized sessions
            var sessions = _this.storage.getSessions(); // .map(session => Immutable.set(session, 'selectedViewer', 'ADMIN'))
            var urlSession = _this.getUrlSession(sessions);
            if (urlSession) {
                if (sessions.length === 1 && sessions[0].query === constants_1.defaultQuery) {
                    return [urlSession];
                }
                return sessions.concat(urlSession);
            }
            if (sessions.length > 0) {
                if (sessions.length === 1 && sessions[0].query === constants_1.defaultQuery) {
                    setTimeout(function () {
                        _this.setCursor({
                            line: 1,
                            ch: 0
                        });
                    }, 5);
                }
                return sessions;
            }
            return [_this.createSession()];
        };
        _this.saveSessions = function () {
            _this.state.sessions.forEach(function (session) {
                return _this.storage.saveSession(Immutable.set(session, 'subscriptionActive', false), false);
            });
        };
        _this.saveHistory = function () {
            _this.storage.syncHistory(_this.state.history);
        };
        _this.handleNewSessionWithoutNewIndexZero = function () {
            return _this.handleNewSession(false);
        };
        _this.createSession = function (session) {
            var newSession;
            var currentActiveSession = _this.state && _this.state.sessions[_this.state.selectedSessionIndex];
            var headers = currentActiveSession ? currentActiveSession.headers : [];
            if (session) {
                newSession = Immutable.set(session, 'id', cuid());
            } else {
                var query = _this.storage.hasExecutedQuery() || _this.state && _this.state.sessions && _this.state.sessions.length > 0 ? '' : constants_1.defaultQuery;
                newSession = Immutable({
                    id: cuid(),
                    selectedViewer: 'ADMIN',
                    query: query,
                    variables: '',
                    result: '',
                    operationName: undefined,
                    hasMutation: false,
                    hasSubscription: false,
                    hasQuery: false,
                    queryTypes: getQueryTypes_1.default(query),
                    starred: false,
                    headers: headers
                });
            }
            _this.storage.saveSession(newSession);
            return newSession;
        };
        _this.createSessionFromQuery = function (query) {
            return Immutable({
                id: cuid(),
                selectedViewer: 'ADMIN',
                query: query,
                variables: '',
                result: '',
                operationName: undefined,
                hasMutation: false,
                hasSubscription: false,
                hasQuery: false,
                queryTypes: getQueryTypes_1.default(query),
                starred: false
            });
        };
        _this.handleChangeHeaders = function (sessionId, headers) {
            _this.setValueInSession(sessionId, 'headers', headers);
        };
        _this.handleViewerChange = function (sessionId, viewer) {
            var handleUser = function handleUser() {
                var session = _this.state.sessions.find(function (sess) {
                    return sess.id === sessionId;
                });
                if (viewer === 'USER') {
                    // give the user some time to realize whats going on
                    setTimeout(function () {
                        _this.setState({
                            selectUserOpen: true,
                            selectUserSessionId: sessionId
                        });
                    }, 300);
                }
                if (session) {
                    _this.resetSubscription(session);
                } else {
                    throw new Error('session not found for viewer change');
                }
            };
            _this.setValueInSession(sessionId, 'selectedViewer', viewer, function () {
                var session = _this.state.sessions.find(function (sess) {
                    return sess.id === sessionId;
                });
                var headers = session.headers ? session.headers.filter(function (h) {
                    return h.name !== 'Authorization';
                }) : [];
                if (viewer === 'ADMIN') {
                    headers = headers.concat({
                        name: 'Authorization',
                        value: "Bearer " + _this.props.adminAuthToken
                    });
                }
                if (viewer === 'ADMIN' || viewer === 'EVERYONE') {
                    console.log('CRAZY');
                    _this.setValueInSession(sessionId, 'headers', headers, function () {
                        handleUser();
                    });
                } else {
                    handleUser();
                }
            });
        };
        _this.handleCloseSelectUser = function () {
            _this.setState({
                selectUserOpen: false
            });
        };
        _this.handleVariableChange = function (sessionId, variables) {
            _this.setValueInSession(sessionId, 'variables', variables);
        };
        _this.handleOperationNameChange = function (sessionId, operationName) {
            _this.setValueInSession(sessionId, 'operationName', operationName);
        };
        _this.cancelSubscription = function (session) {
            _this.setValueInSession(session.id, 'subscriptionActive', false);
            if (session.subscriptionId) {
                if (_this.wsConnections[session.id]) {
                    _this.wsConnections[session.id].unsubscribe(session.subscriptionId);
                }
                _this.setValueInSession(session.id, 'subscriptionId', null);
            }
        };
        _this.permissionFetcher = function (session, graphQLParams) {
            var query = graphQLParams.query;
            if (!query.includes('IntrospectionQuery')) {
                if (!_this.historyIncludes(session)) {
                    setImmediate(function () {
                        _this.addToHistory(session);
                    });
                }
            }
            var endpoint = _this.getPermissionEndpoint();
            var headers = {
                'Content-Type': 'application/json',
                'x-graphcool-source': 'console:playground',
                Authorization: "Bearer " + _this.props.adminAuthToken
            };
            return fetch(endpoint, {
                method: 'post',
                headers: headers,
                body: JSON.stringify(graphQLParams)
            }).then(function (response) {
                _this.storage.executedQuery();
                return response.json();
            });
        };
        _this.fetcher = function (session, graphQLParams) {
            var query = graphQLParams.query,
                operationName = graphQLParams.operationName;
            if (!query.includes('IntrospectionQuery')) {
                if (['STEP3_RUN_QUERY1', 'STEP3_RUN_MUTATION1', 'STEP3_RUN_MUTATION2', 'STEP3_RUN_QUERY2'].indexOf(_this.props.onboardingStep || '') > -1 && typeof _this.props.nextStep === 'function') {
                    if (_this.props.onboardingStep === 'STEP3_RUN_QUERY2') {
                        setTimeout(function () {
                            // typescript wants to be happy...
                            if (typeof _this.props.nextStep === 'function') {
                                _this.props.nextStep();
                            }
                        }, 2000);
                    } else {
                        _this.props.nextStep();
                    }
                }
                if (!_this.historyIncludes(session)) {
                    setImmediate(function () {
                        _this.addToHistory(session);
                    });
                }
                if (isQuerySubscription_1.default(query, operationName)) {
                    /* tslint:disable-next-line */
                    return Observable_1.Observable.create(function (observer) {
                        _this.observers[session.id] = observer;
                        if (!session.subscriptionActive) {
                            _this.setValueInSession(session.id, 'subscriptionActive', true);
                        }
                        var wsConnection = _this.wsConnections[session.id];
                        var id = wsConnection.subscribe(graphQLParams, function (err, res) {
                            var data = { data: res, isSubscription: true };
                            if (err) {
                                data.error = err;
                            }
                            observer.next(data);
                        });
                        _this.setValueInSession(session.id, 'subscriptionId', id);
                        return function () {
                            _this.cancelSubscription(session);
                        };
                    });
                }
            }
            var endpoint = _this.getSimpleEndpoint();
            var headers = {
                'Content-Type': 'application/json',
                'x-graphcool-source': 'console:playground'
            };
            if (session.headers) {
                session.headers.forEach(function (header) {
                    headers[header.name] = header.value;
                });
            }
            return fetch(endpoint, {
                // tslint:disable-line
                method: 'post',
                headers: headers,
                credentials: 'include',
                body: JSON.stringify(graphQLParams)
            }).then(function (response) {
                if (typeof _this.props.onSuccess === 'function') {
                    _this.props.onSuccess(graphQLParams, response);
                }
                if (_this.props.isEndpoint) {
                    history.pushState({}, 'Graphcool Playground', "?query=" + encodeURIComponent(query));
                }
                _this.storage.executedQuery();
                return response.json();
            });
        };
        _this.toggleUseVim = function () {
            _this.setState(function (state) {
                return __assign({}, state, { useVim: !state.useVim });
            }, function () {
                localStorage.setItem('useVim', String(_this.state.useVim));
            });
        };
        _this.toggleShareAllTabs = function () {
            _this.setState(function (state) {
                return __assign({}, state, { shareAllTabs: !state.shareAllTabs });
            });
        };
        _this.toggleShareHTTPHeaders = function () {
            _this.setState(function (state) {
                return __assign({}, state, { shareHttpHeaders: !state.shareHttpHeaders });
            });
        };
        _this.toggleShareHistory = function () {
            _this.setState(function (state) {
                return __assign({}, state, { shareHistory: !state.shareHistory });
            });
        };
        _this.share = function () {
            _this.saveSessions();
            _this.saveHistory();
            _this.storage.saveProject();
            var sharingProject = _this.storage.project;
            if (!_this.state.shareHttpHeaders) {
                sharingProject = __assign({}, sharingProject, { sessions: lodash_1.mapValues(sharingProject.sessions, function (session) {
                        session.headers = [];
                        return session;
                    }) });
            }
            if (!_this.state.shareAllTabs) {
                var currentSession = _this.state.sessions[_this.state.selectedSessionIndex];
                sharingProject = __assign({}, sharingProject, { sessions: (_a = {}, _a[currentSession.id] = currentSession, _a) });
            }
            if (!_this.state.shareHistory) {
                sharingProject = __assign({}, sharingProject, { history: [] });
            }
            _this.props.share(sharingProject);
            _this.setState({ changed: false });
            var _a;
        };
        _this.storage = new PlaygroundStorage_1.default(props.endpoint);
        if (props.session) {
            _this.storage.setState(props.session);
        }
        var sessions = _this.initSessions();
        var selectedSessionIndex = parseInt(_this.storage.getItem('selectedSessionIndex'), 10) || 0;
        _this.state = {
            schema: null,
            schemaCache: null,
            userFields: [],
            sessions: sessions,
            selectedSessionIndex: selectedSessionIndex < sessions.length && selectedSessionIndex > -1 ? selectedSessionIndex : 0,
            historyOpen: false,
            history: _this.storage.getHistory(),
            adminAuthToken: props.adminAuthToken && props.adminAuthToken.length > 0 && props.adminAuthToken || localStorage.getItem('token'),
            selectUserOpen: false,
            selectUserSessionId: undefined,
            codeGenerationPopupOpen: false,
            disableQueryHeader: false,
            theme: localStorage.getItem('theme') || 'dark',
            autoReloadSchema: false,
            useVim: localStorage.getItem('useVim') === 'true' || false,
            shareAllTabs: true,
            shareHttpHeaders: true,
            shareHistory: true,
            changed: false,
            response: undefined,
            userModelName: 'User'
        };
        if ((typeof window === "undefined" ? "undefined" : _typeof(window)) === 'object') {
            window.addEventListener('beforeunload', function () {
                _this.componentWillUnmount();
            });
        }
        ;
        global.p = _this;
        _this.fetcher = _this.fetcher.bind(_this);
        if (typeof _this.props.getRef === 'function') {
            _this.props.getRef(_this);
        }
        return _this;
    }
    Playground.prototype.componentWillMount = function () {
        // look, if there is a session. if not, initiate one.
        this.fetchSchemas().then(this.initSessions);
    };
    Playground.prototype.componentDidMount = function () {
        if (this.initialIndex > -1) {
            this.setState({
                selectedSessionIndex: this.initialIndex
            });
        }
        if (['STEP3_UNCOMMENT_DESCRIPTION', 'STEP3_OPEN_PLAYGROUND'].indexOf(this.props.onboardingStep || '') > -1) {
            this.setCursor({ line: 3, ch: 6 });
        }
        this.initWebsockets();
        if (this.props.adminAuthToken) {
            this.fetchPermissionSchema();
            this.fetchServiceInformation();
        }
    };
    Playground.prototype.componentDidUpdate = function (prevProps) {
        var _this = this;
        if (prevProps.endpoint !== this.props.endpoint || prevProps.adminAuthToken !== this.props.adminAuthToken || prevProps.subscriptionsEndpoint !== this.props.subscriptionsEndpoint) {
            this.saveSessions();
            this.saveHistory();
            this.storage.saveProject();
            this.storage = new PlaygroundStorage_1.default(this.props.endpoint);
            var sessions = this.initSessions();
            this.setState({
                sessions: sessions,
                history: this.storage.getHistory()
            }, function () {
                _this.resetSubscriptions();
                _this.fetchSchemas().then(_this.initSessions);
            });
        }
        if (this.state.autoReloadSchema && !this.schemaReloadInterval) {
            this.fetchSchemas();
            this.schemaReloadInterval = window.setInterval(function () {
                _this.fetchSchemas();
            }, 4000);
        }
        if (!this.state.autoReloadSchema && this.schemaReloadInterval) {
            window.clearInterval(this.schemaReloadInterval);
            this.schemaReloadInterval = null;
        }
    };
    Playground.prototype.componentWillUnmount = function () {
        this.storage.setItem('selectedSessionIndex', String(this.state.selectedSessionIndex));
        this.saveSessions();
        this.saveHistory();
        this.storage.saveProject();
    };
    Playground.prototype.initWebsockets = function () {
        var _this = this;
        this.state.sessions.forEach(function (session) {
            return _this.setWS(session);
        });
    };
    Playground.prototype.setCursor = function (position) {
        var editor = this.graphiqlComponents[this.state.selectedSessionIndex].queryEditorComponent.editor;
        editor.setCursor(position);
    };
    Playground.prototype.renewStack = function (schema) {
        var _this = this;
        var rootMap = utils_1.getRootMap(schema);
        var stacks = this.props.navStack.map(function (stack) {
            return _this.getNewStack(rootMap, schema, stack);
        }).filter(function (s) {
            return s;
        });
        this.props.setStacks(stacks);
    };
    Playground.prototype.getNewStack = function (root, schema, stack) {
        var path = stack.field.path;
        var splittedPath = path.split('/');
        var pointer = null;
        var count = 0;
        var lastPointer = null;
        var y = -1;
        var _loop_1 = function _loop_1() {
            var currentPath = splittedPath.shift();
            if (count === 0) {
                pointer = root[currentPath];
                y = Object.keys(root).indexOf(currentPath);
            } else {
                var argFound = pointer.args.find(function (arg) {
                    return arg.name === currentPath;
                });
                lastPointer = pointer;
                if (argFound) {
                    pointer = argFound;
                } else {
                    if (pointer.type.ofType) {
                        pointer = utils_1.getDeeperType(pointer.type.ofType);
                    }
                    if (pointer.type) {
                        pointer = pointer.type;
                    }
                    pointer = pointer.getFields()[currentPath] || pointer.getInterfaces().find(function (i) {
                        return i.name === currentPath;
                    });
                }
            }
            if (lastPointer) {
                y = utils_1.getElementIndex(schema, lastPointer, pointer);
            }
            count++;
        };
        while (splittedPath.length > 0) {
            _loop_1();
        }
        if (!pointer) {
            return null;
        }
        pointer.path = path;
        pointer.parent = lastPointer;
        return __assign({}, stack, { y: y, field: pointer });
    };
    Playground.prototype.extractUserField = function (simpleSchema, userModelName) {
        var modelName = userModelName || this.state.userModelName;
        var userSchema = simpleSchema.getType(modelName);
        if (userSchema) {
            var userSchemaFields_1 = userSchema.getFields();
            var userFields = Object.keys(userSchemaFields_1).map(function (fieldName) {
                return userSchemaFields_1[fieldName];
            }).filter(function (field) {
                // filter password, meta fields and relation fields
                return field.name[0] !== '_' && field.name !== 'password' && !(field.type instanceof graphql_1.GraphQLList || field.type instanceof graphql_1.GraphQLObjectType);
            });
            // put id to beginning
            userFields.sort(function (a, b) {
                if (a.name === 'id') {
                    return -1;
                }
                if (b.name === 'id') {
                    return 1;
                }
                return a.name > b.name ? 1 : -1;
            });
            return userFields.map(function (field) {
                var size = calculate_size_1.default(field.name, {
                    font: 'Open Sans',
                    fontWeight: '600',
                    fontSize: '16px'
                });
                var width = size.width;
                if (field.name === 'id') {
                    width = 220;
                }
                // TODO create type to field width map
                field.width = Math.max(width, 185) + 50;
                return field;
            });
        } else {
            return [];
        }
    };
    Playground.prototype.fetchPermissionSchema = function () {
        var _this = this;
        var headers = {
            Authorization: "Bearer " + this.props.adminAuthToken
        };
        this.fetchSchema(this.getPermissionEndpoint(), headers).then(function (schema) {
            var permissionSchema = graphql_1.buildClientSchema(schema.data);
            _this.setState({
                permissionSchema: permissionSchema
            });
        }).catch(function (error) {
            /* tslint:disable-next-line */
            console.error(error);
        });
    };
    Playground.prototype.fetchServiceInformation = function () {
        var _this = this;
        var systemApi = this.getSystemEndpoint();
        var id = this.extractServiceId();
        fetch(systemApi, {
            method: 'post',
            body: JSON.stringify({
                query: constants_2.serviceInformationQuery,
                variables: { id: id }
            }),
            headers: {
                'Content-Type': 'application/json',
                Authorization: "Bearer " + this.state.adminAuthToken
            }
        }).then(function (res) {
            return res.json();
        }).then(function (res) {
            if (res.data.viewer.project) {
                var _a = res.data.viewer.project,
                    models = _a.models,
                    relations = _a.relations;
                _this.setState({
                    serviceInformation: {
                        relations: relations.edges.map(function (edge) {
                            return edge.node;
                        }),
                        models: models.edges.map(function (edge) {
                            return edge.node;
                        })
                    },
                    userModelName: models.edges[0].node.name
                });
            } else {
                /* tslint:disable-next-line */
                console.error('Error while fetching service information', res);
            }
        });
    };
    Playground.prototype.getModelNames = function () {
        if (this.state.serviceInformation) {
            return this.state.serviceInformation.models.map(function (m) {
                return m.name;
            });
        }
        return [];
    };
    Playground.prototype.extractServiceId = function () {
        return this.props.endpoint.split('/').slice(-1)[0];
    };
    Playground.prototype.getPermissionEndpoint = function () {
        return this.props.endpoint + '/permissions';
    };
    Playground.prototype.fetchSchema = function (endpointUrl, headers) {
        var _this = this;
        if (headers === void 0) {
            headers = {};
        }
        return fetch(endpointUrl, {
            method: 'post',
            headers: __assign({ 'Content-Type': 'application/json', 'x-graphcool-source': 'console:playground' }, headers),
            body: JSON.stringify({ query: constants_1.introspectionQuery })
        }).then(function (response) {
            return response.json();
        }).catch(function (e) {
            _this.setState({
                response: {
                    date: "Error: Could not fetch schema from " + endpointUrl + ". Make sure the url is correct.",
                    time: new Date()
                }
            });
        });
    };
    Playground.prototype.render = function () {
        var _this = this;
        var _a = this.state,
            sessions = _a.sessions,
            selectedSessionIndex = _a.selectedSessionIndex,
            theme = _a.theme;
        var isEndpoint = this.props.isEndpoint;
        // {
        //   'blur': this.state.historyOpen,
        // },
        if (this.state.selectUserOpen && !this.props.adminAuthToken) {
            throw new Error('The "Select User" Popup is open, but no admin token is provided.');
        }
        var selectedEndpointUrl = isEndpoint ? location.href : this.getSimpleEndpoint();
        var isGraphcoolUrl = this.isGraphcoolUrl(selectedEndpointUrl);
        return React.createElement(
            ThemeProvider_1.default,
            { theme: this.state.theme },
            React.createElement(
                "div",
                { className: cx('playground'), "data-jsx": 4176650908
                },
                React.createElement(_style2.default, {
                    styleId: 4176650908,
                    css: ".playground[data-jsx=\"4176650908\"] {letter-spacing: 0.53px;}.blur[data-jsx=\"4176650908\"] {filter: url('data:image/svg+xml;charset=utf-8,<svg xmlns=\"http://www.w3.org/2000/svg\"><filter id=\"filter\"><feGaussianBlur stdDeviation=\"5\" /></filter></svg>#filter');-webkit-filter: blur(5px);filter: blur(5px);}.graphiqls-container[data-jsx=\"4176650908\"] {height: calc(100vh - 57px);}.h100,\n.playground[data-jsx=\"4176650908\"],\n.graphiql-wrapper[data-jsx=\"4176650908\"] {height: 100%;}.flex,\n.playground[data-jsx=\"4176650908\"] {display: -ms-flexbox;display: flex;}.flexColumn,\n.playground[data-jsx=\"4176650908\"] {-ms-flex-direction: column;flex-direction: column;}.relative,\n.graphiqls-container[data-jsx=\"4176650908\"],\n.graphiql-wrapper[data-jsx=\"4176650908\"] {position: relative;}.overflowHidden,\n.graphiqls-container[data-jsx=\"4176650908\"] {overflow: hidden;}.w100,\n.graphiql-wrapper[data-jsx=\"4176650908\"] {width: 100%;}.h100,\n.playground[data-jsx=\"4176650908\"],\n.graphiql-wrapper[data-jsx=\"4176650908\"] {height: 100%;}.relative,\n.graphiqls-container[data-jsx=\"4176650908\"],\n.graphiql-wrapper[data-jsx=\"4176650908\"] {position: relative;}"
                }),
                React.createElement(TabBar_1.TabBar, { sessions: sessions, selectedSessionIndex: selectedSessionIndex, onNewSession: this.handleNewSessionWithoutNewIndexZero, onCloseSession: this.handleCloseSession, onOpenHistory: this.handleOpenHistory, onSelectSession: this.handleSelectSession, onboardingStep: this.props.onboardingStep, nextStep: this.props.nextStep, tether: this.props.tether, isApp: this.props.isApp }),
                React.createElement(
                    "div",
                    { className: cx('graphiqls-container', {
                            'docs-graphiql': theme === 'light'
                        }), "data-jsx": 4176650908
                    },
                    sessions.map(function (session, index) {
                        return React.createElement(
                            "div",
                            { key: session.id, className: cx('graphiql-wrapper', {
                                    active: index === selectedSessionIndex
                                }), style: {
                                    top: "-" + 100 * selectedSessionIndex + "%"
                                }, "data-jsx": 4176650908
                            },
                            React.createElement(GraphQLEditorSession_1.default, { key: session.id, session: session, index: index, schemaCache: session.permission ? _this.state.permissionSchema : _this.state.schemaCache, isGraphcoolUrl: isGraphcoolUrl, fetcher: session.permission ? _this.permissionFetcher : _this.fetcher, adminAuthToken: _this.props.adminAuthToken, isEndpoint: Boolean(isEndpoint), storage: _this.storage.getSessionStorage(session.id), onClickCodeGeneration: _this.handleClickCodeGeneration, onChangeViewer: _this.handleViewerChange, onEditOperationName: _this.handleOperationNameChange, onEditVariables: _this.handleVariableChange, onEditQuery: _this.handleQueryChange, onChangeHeaders: _this.handleChangeHeaders, responses: _this.state.response ? [_this.state.response] : undefined, disableQueryHeader: _this.state.disableQueryHeader, onboardingStep: index === selectedSessionIndex ? _this.props.onboardingStep : undefined, tether: _this.props.tether, nextStep: _this.props.nextStep, onRef: _this.setRef, autofillMutation: _this.autofillMutation, useVim: _this.state.useVim && index === selectedSessionIndex, isActive: index === selectedSessionIndex, permission: session.permission, serviceInformation: _this.state.serviceInformation })
                        );
                    })
                ),
                React.createElement(Settings_1.default, { onToggleTheme: this.toggleTheme, theme: this.state.theme, autoReload: this.state.autoReloadSchema, onToggleReload: this.toggleSchemaReload, onReload: this.fetchSchemas, endpoint: this.props.endpoint, onChangeEndpoint: this.props.onChangeEndpoint, useVim: this.state.useVim, onToggleUseVim: this.toggleUseVim, subscriptionsEndpoint: this.props.subscriptionsEndpoint || '', onChangeSubscriptionsEndpoint: this.props.onChangeSubscriptionsEndpoint }),
                this.props.adminAuthToken && this.state.serviceInformation && React.createElement(NewPermissionTab_1.default, { serviceInformation: this.state.serviceInformation, theme: this.state.theme, onNewPermissionTab: this.handleNewPermissionTab }),
                React.createElement(Share_1.default, { theme: this.state.theme, onShare: this.share, onToggleHistory: this.toggleShareHistory, onToggleAllTabs: this.toggleShareAllTabs, onToggleHttpHeaders: this.toggleShareHTTPHeaders, history: this.state.shareHistory, allTabs: this.state.shareAllTabs, httpHeaders: this.state.shareHttpHeaders, shareUrl: this.props.shareUrl, reshare: this.state.changed }),
                React.createElement(GraphDocs_1.default, { schema: this.state.schemaCache }),
                this.state.historyOpen && React.createElement(HistoryPopup_1.default, { isOpen: this.state.historyOpen, onRequestClose: this.handleCloseHistory, historyItems: this.state.history, onItemStarToggled: this.handleItemStarToggled, fetcherCreater: this.fetcher, schema: this.state.schemaCache, onCreateSession: this.handleCreateSession, isGraphcool: isGraphcoolUrl }),
                this.props.adminAuthToken && this.state.selectUserOpen && this.renderUserPopup(),
                this.state.codeGenerationPopupOpen && this.renderCodeGenerationPopup()
            )
        );
    };
    Playground.prototype.renderUserPopup = function () {
        return React.createElement(SelectUserPopup_1.default, { isOpen: this.state.selectUserOpen, onRequestClose: this.handleCloseSelectUser, adminAuthToken: this.props.adminAuthToken, userFields: this.state.userFields, onSelectUser: this.handleUserSelection, endpointUrl: this.getSimpleEndpoint(), modelNames: this.getModelNames(), userModelName: this.state.userModelName, onChangeUserModelName: this.handleUserModelChange });
    };
    Playground.prototype.renderCodeGenerationPopup = function () {
        var _a = this.state,
            sessions = _a.sessions,
            selectedSessionIndex = _a.selectedSessionIndex;
        var isEndpoint = this.props.isEndpoint;
        var selectedSession = sessions[selectedSessionIndex];
        var selectedEndpointUrl = isEndpoint ? location.href : this.getSimpleEndpoint();
        return React.createElement(CodeGenerationPopup_1.default, { endpointUrl: selectedEndpointUrl, isOpen: this.state.codeGenerationPopupOpen, onRequestClose: this.handleCloseCodeGeneration, query: selectedSession.query });
    };
    Playground.prototype.getDefaultPermissionQuery = function (session) {
        var modelName = session.relationName ? this.state.serviceInformation.relations.find(function (r) {
            return r.name === session.relationName;
        }).leftModel.name : session.modelName;
        var operationName = session.relationName || "permit" + session.modelName;
        return "query " + operationName + " {\n  Some" + modelName + "Exists\n}";
    };
    Playground.prototype.setValueInSession = function (sessionId, key, value, cb) {
        this.setState(function (state) {
            // TODO optimize the lookup with a lookup table
            var i = state.sessions.findIndex(function (s) {
                return s.id === sessionId;
            });
            return __assign({}, state, { sessions: Immutable.setIn(state.sessions, [i, key], value), changed: true });
        }, function () {
            if (typeof cb === 'function') {
                cb();
            }
        });
    };
    Playground.prototype.resetSubscriptions = function () {
        var _this = this;
        this.state.sessions.forEach(function (session) {
            return _this.resetSubscription(session);
        });
    };
    Playground.prototype.resetSubscription = function (session) {
        if (this.observers[session.id]) {
            this.observers[session.id].complete();
            delete this.observers[session.id];
        }
        this.cancelSubscription(session);
        this.setWS(session);
    };
    Playground.prototype.getUrlSession = function (sessions) {
        var prefix = '?query=';
        if (location.search.includes(prefix)) {
            var uri = location.search.slice(prefix.length, location.search.length);
            var query_1 = decodeURIComponent(uri);
            var equivalent = sessions.findIndex(function (session) {
                return session.query.trim() === query_1.trim();
            });
            if (equivalent > -1) {
                this.initialIndex = equivalent;
            } else {
                return this.createSessionFromQuery(query_1);
            }
        }
        return null;
    };
    Playground.prototype.initOnboardingSessions = function () {
        var session = this.createSession();
        return [__assign({}, session, { query: data_1.onboardingQuery1 })];
    };
    Playground.prototype.setValueInHistory = function (sessionId, key, value) {
        this.setState(function (state) {
            // TODO optimize the lookup with a lookup table
            var i = state.history.findIndex(function (s) {
                return s.id === sessionId;
            });
            return __assign({}, state, { history: Immutable.setIn(state.history, [i, key], value) });
        });
    };
    Playground.prototype.isGraphcoolUrl = function (endpoint) {
        return endpoint.includes('api.graph.cool');
    };
    Playground.prototype.getSimpleEndpoint = function () {
        if (this.props.isEndpoint) {
            return location.pathname;
        }
        return this.props.endpoint;
    };
    Playground.prototype.getSystemEndpoint = function () {
        return this.httpApiPrefix + "/system";
    };
    Object.defineProperty(Playground.prototype, "httpApiPrefix", {
        get: function get() {
            return this.props.endpoint.match(/(https?:\/\/.*?)\//)[1];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Playground.prototype, "wsApiPrefix", {
        get: function get() {
            var endpoint = this.props.endpoint;
            var isDev = endpoint.indexOf('dev.graph.cool') > -1;
            var isLocalhost = endpoint.indexOf('localhost') > -1;
            // tslint:disable-next-line
            if (isLocalhost) {
                return 'ws://localhost:8085/v1';
            } else if (isDev) {
                return 'wss://dev.subscriptions.graph.cool/v1';
            } else {
                return 'wss://subscriptions.graph.cool/v1';
            }
        },
        enumerable: true,
        configurable: true
    });
    Playground.prototype.getWSEndpoint = function () {
        if (this.props.subscriptionsEndpoint) {
            return this.props.subscriptionsEndpoint;
        }
        var projectId = this.props.projectId || this.props.endpoint.includes('graph.cool') && this.props.endpoint.split('/').slice(-1)[0];
        return this.wsApiPrefix + "/" + projectId;
    };
    Playground.prototype.addToHistory = function (session) {
        var id = cuid();
        var historySession = Immutable.merge(session, {
            id: id,
            date: new Date()
        });
        this.setState(function (state) {
            return __assign({}, state, { history: [historySession].concat(state.history) });
        });
        this.storage.addToHistory(historySession);
    };
    Playground.prototype.historyIncludes = function (session) {
        var duplicate = this.state.history.find(function (item) {
            return session.query === item.query && session.variables === item.variables && session.operationName === item.operationName && session.selectedViewer === item.selectedViewer;
        });
        return Boolean(duplicate);
    };
    return Playground;
}(React.PureComponent);
exports.Playground = Playground;
exports.default = react_redux_1.connect(function (state) {
    return state.graphiqlDocs;
}, {
    setStacks: graphiql_docs_1.setStacks
})(Playground);
//# sourceMappingURL=Playground.jsx.map