/// <reference types="react" />
import * as React from 'react';
import { GraphQLSchema } from 'graphql/type/schema';
import { OperationDefinition, PermissionSession, ServiceInformation, Viewer } from '../../types';
import { Response } from '../Playground';
import { Header } from './HttpHeaders';
/**
 * The top-level React component for GraphQLEditor, intended to encompass the entire
 * browser viewport.
 */
export interface Props {
    fetcher: (params: any) => Promise<any>;
    isGraphcoolUrl?: boolean;
    schema?: GraphQLSchema;
    query?: string;
    variables?: string;
    operationName?: string;
    responses?: Response[];
    isActive: boolean;
    storage?: any;
    defaultQuery?: string;
    onEditQuery?: (data: any) => void;
    onEditVariables?: (variables: any) => any;
    onEditOperationName?: (name: any) => any;
    onToggleDocs?: (value: boolean) => any;
    onClickCodeGeneration?: any;
    onChangeHeaders?: (headers: Header[]) => any;
    getDefaultFieldNames?: () => any;
    autofillMutation?: () => void;
    onChangeViewer?: (data: any) => void;
    headers?: any[];
    showViewAs?: boolean;
    showSelectUser?: boolean;
    showCodeGeneration?: boolean;
    showEndpoints?: boolean;
    showQueryTitle?: boolean;
    showResponseTitle?: boolean;
    showDownloadJsonButton?: boolean;
    disableQueryHeader?: boolean;
    selectedViewer?: Viewer;
    queryOnly?: boolean;
    showDocs?: boolean;
    rerenderQuery?: boolean;
    operations?: OperationDefinition[];
    showSchema?: boolean;
    schemaIdl?: string;
    schemaModelName?: string;
    disableAutofocus?: boolean;
    disableResize?: boolean;
    onboardingStep?: any;
    tether?: any;
    nextStep?: () => void;
    disableAnimation?: boolean;
    hideLineNumbers?: boolean;
    hideGutters?: boolean;
    readonly?: boolean;
    useVim?: boolean;
    permission?: PermissionSession;
    serviceInformation?: ServiceInformation;
}
export interface State {
    schema: any;
    query: any;
    variables: any;
    operationName: string;
    responses: any[];
    editorFlex: number;
    variableEditorOpen: boolean;
    variableEditorHeight: number;
    docExploreOpen: boolean;
    docExplorerWidth: number;
    isWaitingForReponse: boolean;
    subscription: any;
    variableToType: any;
    operations: any[];
    docExplorerOpen: boolean;
    schemaExplorerOpen: boolean;
    schemaExplorerWidth: number;
    isWaitingForResponse: boolean;
    selectedVariableNames: string[];
}
export interface SimpleProps {
    children?: any;
}
export interface ToolbarButtonProps extends SimpleProps {
    onClick: (e: any) => void;
    title: string;
    label: string;
}
export declare class GraphQLEditor extends React.PureComponent<Props, State> {
    static Logo: (props: SimpleProps) => JSX.Element;
    static Toolbar: (props: SimpleProps) => JSX.Element;
    static Footer: (props: SimpleProps) => JSX.Element;
    static ToolbarButton: (props: ToolbarButtonProps) => JSX.Element;
    codeMirrorSizer: any;
    queryEditorComponent: any;
    variableEditorComponent: any;
    resultComponent: any;
    editorBarComponent: any;
    docExplorerComponent: any;
    private storage;
    private editorQueryID;
    private resultID;
    private reflectQueryVariablesToUI;
    private updateQueryFacts;
    constructor(props: any);
    componentDidMount(): void;
    componentWillReceiveProps(nextProps: any): void;
    componentDidUpdate(): void;
    componentWillUnmount(): void;
    render(): JSX.Element;
    tetherClick: () => void;
    setEditorBarComponent: (ref: any) => void;
    setQueryEditorComponent: (ref: any) => void;
    setVariableEditorComponent: (ref: any) => void;
    setResultComponent: (ref: any) => void;
    /**
     * Inspect the query, automatically filling in selection sets for non-leaf
     * fields which do not yet have them.
     *
     * @public
     */
    autoCompleteLeafs(): any;
    _ensureOfSchema(): void;
    _storageGet: (name: string) => any;
    _storageSet: (name: string, value: any) => void;
    _fetchQuery(query: any, variables: any, operationName: any, cb: any): any;
    handleRunQuery: (selectedOperationName: any) => void;
    handleStopQuery: () => void;
    _runQueryAtCursor(): void;
    handlePrettifyQuery: () => void;
    handleEditQuery: (value: any) => void | null;
    handleEditVariables: (value: any) => void;
    handleHintInformationRender: (elem: any) => void;
    handleEditorRunQuery: () => void;
    handleToggleDocs: () => void;
    handleToggleSchema: () => void;
    handleResizeStart: (downEvent: any) => void;
    _didClickDragBar(event: any): boolean;
    handleDocsResizeStart: (downEvent: any) => void;
    handleSchemaResizeStart: (downEvent: any) => void;
    handleVariableResizeStart: (downEvent: any) => void;
    handleDownloadJSON: () => void;
    private onClickHintInformation;
    private toggleVariableSelection;
    private getSelectedVariables();
    private getPermissionQueryArguments();
    private getVariables();
}
