"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var ReactDOM = require("react-dom");
var graphql_1 = require("graphql");
var cn = require("classnames");
var ExecuteButton_1 = require("./ExecuteButton");
var ToolbarButton_1 = require("graphiql/dist/components/ToolbarButton");
var QueryEditor_1 = require("./QueryEditor");
var VariableEditor_1 = require("graphiql/dist/components/VariableEditor");
var CodeMirrorSizer_1 = require("graphiql/dist/utility/CodeMirrorSizer");
var getQueryFacts_1 = require("graphiql/dist/utility/getQueryFacts");
var getSelectedOperationName_1 = require("graphiql/dist/utility/getSelectedOperationName");
var debounce_1 = require("graphiql/dist/utility/debounce");
var find_1 = require("graphiql/dist/utility/find");
var fillLeafs_1 = require("graphiql/dist/utility/fillLeafs");
var elementPosition_1 = require("graphiql/dist/utility/elementPosition");
var introspectionQueries_1 = require("graphiql/dist/utility/introspectionQueries");
var index_1 = require("./util/index");
var QueryHeader_1 = require("./QueryHeader");
var ResultHeader_1 = require("./ResultHeader");
var ResultViewer_1 = require("./ResultViewer");
var ageOfDate_1 = require("./util/ageOfDate");
var HttpHeaders_1 = require("./HttpHeaders");
var constants_1 = require("../../constants");
var Spinner_1 = require("../Spinner");
var PermissionVariables_1 = require("./PermissionVariables");
var ast_1 = require("./ast");
var lodash_1 = require("lodash");
// tslint:disable-next-line
var CSSTransitionGroup = require('react-transition-group/CSSTransitionGroup');
var GraphQLEditor = /** @class */ (function (_super) {
    __extends(GraphQLEditor, _super);
    function GraphQLEditor(props) {
        var _this = _super.call(this, props) || this;
        _this.resultID = 0;
        _this.reflectQueryVariablesToUI = debounce_1.default(150, function (query) {
            var variables = ast_1.getVariableNamesFromQuery(query, true, _this.props.schema).variables;
            _this.setState({
                selectedVariableNames: variables,
            });
        });
        _this.updateQueryFacts = debounce_1.default(150, function (query) {
            var queryFacts = getQueryFacts_1.default(_this.state.schema, query);
            if (queryFacts) {
                // Update operation name should any query names change.
                var operationName = getSelectedOperationName_1.default(_this.state.operations, _this.state.operationName, queryFacts.operations);
                // Report changing of operationName if it changed.
                var onEditOperationName = _this.props.onEditOperationName;
                if (onEditOperationName && operationName !== _this.state.operationName) {
                    onEditOperationName(operationName);
                }
                _this.setState(__assign({ operationName: operationName }, queryFacts));
            }
        });
        _this.tetherClick = function () {
            if (_this.props.onboardingStep.startsWith('STEP3_ENTER_MUTATION') &&
                typeof _this.props.autofillMutation === 'function') {
                _this.props.autofillMutation();
            }
        };
        _this.setEditorBarComponent = function (ref) {
            _this.editorBarComponent = ref;
        };
        _this.setQueryEditorComponent = function (ref) {
            _this.queryEditorComponent = ref;
        };
        _this.setVariableEditorComponent = function (ref) {
            _this.variableEditorComponent = ref;
        };
        _this.setResultComponent = function (ref) {
            _this.resultComponent = ref;
        };
        // tslint:disable-next-line
        _this._storageGet = function (name) {
            if (_this.storage) {
                var value = _this.storage.getItem('graphiql:' + name);
                // Clean up any inadvertently saved null/undefined values.
                if (value === 'null' || value === 'undefined') {
                    _this.storage.removeItem('graphiql:' + name);
                }
                else {
                    return value;
                }
            }
        };
        // tslint:disable-next-line
        _this._storageSet = function (name, value) {
            if (_this.storage) {
                if (value !== undefined) {
                    _this.storage.setItem('graphiql:' + name, value);
                }
                else {
                    _this.storage.removeItem('graphiql:' + name);
                }
            }
        };
        _this.handleRunQuery = function (selectedOperationName) {
            _this.editorQueryID++;
            var queryID = _this.editorQueryID;
            // Use the edited query after autoCompleteLeafs() runs or,
            // in case autoCompletion fails (the function returns undefined),
            // the current query from the editor.
            var editedQuery = _this.autoCompleteLeafs() || _this.state.query;
            var variables = _this.state.variables;
            var operationName = _this.state.operationName;
            // If an operation was explicitly provided, different from the current
            // operation name, then report that it changed.
            if (selectedOperationName && selectedOperationName !== operationName) {
                operationName = selectedOperationName;
                var onEditOperationName = _this.props.onEditOperationName;
                if (onEditOperationName) {
                    onEditOperationName(operationName);
                }
            }
            try {
                _this.setState({
                    isWaitingForResponse: true,
                    responses: [{ date: null, time: new Date() }],
                    operationName: operationName,
                });
                // _fetchQuery may return a subscription.
                var subscription = _this._fetchQuery(editedQuery, variables, operationName, function (result) {
                    if (queryID === _this.editorQueryID) {
                        var isSubscription = false;
                        if (result.isSubscription) {
                            isSubscription = true;
                            delete result.isSubscription;
                        }
                        var responses = void 0;
                        var response = JSON.stringify(result, null, 2);
                        if (isSubscription) {
                            responses = _this.state.responses
                                .filter(function (res) { return res && res.date; })
                                .concat({
                                date: response,
                                time: new Date(),
                                resultID: _this.resultID++,
                            });
                        }
                        else {
                            responses = [
                                { date: response, time: new Date(), resultID: _this.resultID++ },
                            ];
                        }
                        _this.setState({
                            isWaitingForResponse: false,
                            responses: responses,
                        });
                    }
                });
                _this.setState({ subscription: subscription });
            }
            catch (error) {
                _this.setState({
                    isWaitingForResponse: false,
                    responses: [{ date: error.message, time: new Date() }],
                });
            }
        };
        _this.handleStopQuery = function () {
            var subscription = _this.state.subscription;
            _this.setState({
                isWaitingForResponse: false,
                subscription: null,
            });
            if (subscription) {
                subscription.unsubscribe();
            }
        };
        _this.handlePrettifyQuery = function () {
            var query = graphql_1.print(graphql_1.parse(_this.state.query));
            var editor = _this.queryEditorComponent.getCodeMirror();
            editor.setValue(query);
        };
        _this.handleEditQuery = function (value) {
            if (_this.state.schema) {
                _this.updateQueryFacts(value);
            }
            _this.setState({ query: value });
            if (_this.props.onEditQuery) {
                return _this.props.onEditQuery(value);
            }
            return null;
        };
        _this.handleEditVariables = function (value) {
            _this.setState({ variables: value });
            if (_this.props.onEditVariables) {
                _this.props.onEditVariables(value);
            }
        };
        _this.handleHintInformationRender = function (elem) {
            elem.addEventListener('click', _this.onClickHintInformation);
            var onRemoveFn;
            elem.addEventListener('DOMNodeRemoved', (onRemoveFn = function () {
                elem.removeEventListener('DOMNodeRemoved', onRemoveFn);
                elem.removeEventListener('click', _this.onClickHintInformation);
            }));
        };
        _this.handleEditorRunQuery = function () {
            _this._runQueryAtCursor();
        };
        _this.handleToggleDocs = function () {
            if (typeof _this.props.onToggleDocs === 'function') {
                _this.props.onToggleDocs(!_this.state.docExplorerOpen);
            }
            _this.setState({ docExplorerOpen: !_this.state.docExplorerOpen });
        };
        _this.handleToggleSchema = function () {
            _this.setState({
                schemaExplorerOpen: !_this.state.schemaExplorerOpen,
            });
        };
        _this.handleResizeStart = function (downEvent) {
            if (_this.props.disableResize) {
                return;
            }
            if (!_this._didClickDragBar(downEvent)) {
                return;
            }
            downEvent.preventDefault();
            var offset = downEvent.clientX - elementPosition_1.getLeft(downEvent.target);
            var onMouseMove = function (moveEvent) {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                var editorBar = ReactDOM.findDOMNode(_this.editorBarComponent);
                var leftSize = moveEvent.clientX - elementPosition_1.getLeft(editorBar) - offset;
                var rightSize = editorBar.clientWidth - leftSize;
                _this.setState({ editorFlex: leftSize / rightSize });
            };
            var onMouseUp = function () {
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        _this.handleDocsResizeStart = function (downEvent) {
            downEvent.preventDefault();
            var hadWidth = _this.state.docExplorerWidth;
            var offset = downEvent.clientX - elementPosition_1.getLeft(downEvent.target);
            var onMouseMove = function (moveEvent) {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                var app = ReactDOM.findDOMNode(_this);
                var cursorPos = moveEvent.clientX - elementPosition_1.getLeft(app) - offset;
                var docsSize = app.clientWidth - cursorPos;
                if (docsSize < 100) {
                    _this.setState({ docExplorerOpen: false });
                }
                else {
                    _this.setState({
                        docExplorerOpen: true,
                        docExplorerWidth: Math.min(docsSize, 850),
                    });
                }
            };
            var onMouseUp = function () {
                if (!_this.state.docExplorerOpen) {
                    _this.setState({ docExplorerWidth: hadWidth });
                }
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        _this.handleSchemaResizeStart = function (downEvent) {
            downEvent.preventDefault();
            var hadWidth = _this.state.schemaExplorerWidth;
            var offset = downEvent.clientX - elementPosition_1.getLeft(downEvent.target);
            var onMouseMove = function (moveEvent) {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                var app = ReactDOM.findDOMNode(_this);
                var cursorPos = moveEvent.clientX - elementPosition_1.getLeft(app) - offset;
                var schemaSize = app.clientWidth - cursorPos;
                if (schemaSize < 100) {
                    _this.setState({ schemaExplorerOpen: false });
                }
                else {
                    _this.setState({
                        schemaExplorerOpen: true,
                        schemaExplorerWidth: Math.min(schemaSize, 850),
                    });
                }
            };
            var onMouseUp = function () {
                if (!_this.state.schemaExplorerOpen) {
                    _this.setState({ schemaExplorerWidth: hadWidth });
                }
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        _this.handleVariableResizeStart = function (downEvent) {
            downEvent.preventDefault();
            var didMove = false;
            var wasOpen = _this.state.variableEditorOpen;
            var hadHeight = _this.state.variableEditorHeight;
            var offset = downEvent.clientY - elementPosition_1.getTop(downEvent.target);
            var onMouseMove = function (moveEvent) {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                didMove = true;
                var editorBar = ReactDOM.findDOMNode(_this.editorBarComponent);
                var topSize = moveEvent.clientY - elementPosition_1.getTop(editorBar) - offset;
                var bottomSize = editorBar.clientHeight - topSize;
                if (bottomSize < 60) {
                    _this.setState({
                        variableEditorOpen: false,
                        variableEditorHeight: hadHeight,
                    });
                }
                else {
                    _this.setState({
                        variableEditorOpen: true,
                        variableEditorHeight: bottomSize,
                    });
                }
            };
            var onMouseUp = function () {
                if (!didMove) {
                    _this.setState({ variableEditorOpen: !wasOpen });
                }
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        _this.handleDownloadJSON = function () {
            index_1.download(_this.state.responses[0].date, 'result.json', 'application/json');
        };
        _this.onClickHintInformation = function (event) {
            if (event.target.className === 'typeName') {
                var typeName = event.target.innerHTML;
                var schema = _this.state.schema;
                if (schema) {
                    var type_1 = schema.getType(typeName);
                    if (type_1) {
                        _this.setState({ docExplorerOpen: true }, function () {
                            _this.docExplorerComponent.showDoc(type_1);
                        });
                    }
                }
            }
        };
        _this.toggleVariableSelection = function (variable) {
            _this.setState(function (state) {
                var selectedVariableNames = state.selectedVariableNames;
                if (selectedVariableNames.includes(variable.name)) {
                    var index = selectedVariableNames.indexOf(variable.name);
                    return __assign({}, state, { selectedVariableNames: selectedVariableNames.slice(0, index).concat(selectedVariableNames.slice(index + 1, selectedVariableNames.length)) });
                }
                return __assign({}, state, { selectedVariableNames: selectedVariableNames.concat(variable.name) });
            }, function () {
                var variables = _this.getSelectedVariables();
                var newQuery = ast_1.putVariablesToQuery(_this.state.query, variables);
                _this.setState({ query: newQuery });
                if (typeof _this.props.onEditQuery === 'function') {
                    _this.props.onEditQuery(newQuery);
                }
            });
        };
        // Ensure props are correct
        if (typeof props.fetcher !== 'function') {
            throw new TypeError('GraphQLEditor requires a fetcher function.');
        }
        // Cache the storage instance
        _this.storage =
            props.storage || typeof window !== 'undefined'
                ? window.localStorage
                : {
                    setItem: function () { return null; },
                    removeItem: function () { return null; },
                    getItem: function () { return null; },
                };
        // Determine the initial query to display.
        var query = props.query !== undefined
            ? props.query
            : _this._storageGet('query') !== null
                ? _this._storageGet('query')
                : props.defaultQuery !== undefined ? props.defaultQuery : constants_1.defaultQuery;
        // Get the initial query facts.
        var queryFacts = getQueryFacts_1.default(props.schema, query);
        // Determine the initial variables to display.
        var variables = props.variables !== undefined
            ? props.variables
            : _this._storageGet('variables');
        // Determine the initial operationName to use.
        var operationName = props.operationName !== undefined
            ? props.operationName
            : getSelectedOperationName_1.default(null, _this._storageGet('operationName'), queryFacts && queryFacts.operations);
        // Initialize state
        _this.state = __assign({ schema: props.schema, query: query,
            variables: variables,
            operationName: operationName, responses: props.responses || [], editorFlex: Number(_this._storageGet('editorFlex')) || 1, variableEditorOpen: Boolean(variables), variableEditorHeight: Number(_this._storageGet('variableEditorHeight')) || 200, docExplorerOpen: false, docExplorerWidth: Number(_this._storageGet('docExplorerWidth')) || 350, schemaExplorerOpen: false, schemaExplorerWidth: Number(_this._storageGet('schemaExplorerWidth')) || 350, isWaitingForResponse: false, subscription: null, selectedVariableNames: [] }, queryFacts);
        // Ensure only the last executed editor query is rendered.
        _this.editorQueryID = 0;
        // Subscribe to the browser window closing, treating it as an unmount.
        if (typeof window === 'object') {
            window.addEventListener('beforeunload', function () { return _this.componentWillUnmount(); });
        }
        return _this;
    }
    GraphQLEditor.prototype.componentDidMount = function () {
        // Ensure a form of a schema exists (including `null`) and
        // if not, fetch one using an introspection query.
        this._ensureOfSchema();
        // Utility for keeping CodeMirror correctly sized.
        this.codeMirrorSizer = new CodeMirrorSizer_1.default();
        global.g = this;
        this.reflectQueryVariablesToUI(this.state.query);
    };
    GraphQLEditor.prototype.componentWillReceiveProps = function (nextProps) {
        var nextSchema = this.state.schema;
        var nextQuery = this.state.query;
        var nextVariables = this.state.variables;
        var nextOperationName = this.state.operationName;
        var nextResponses = this.state.responses;
        if (nextProps.schema !== undefined) {
            nextSchema = nextProps.schema;
        }
        if (nextProps.query !== undefined &&
            (this.props.rerenderQuery || nextProps.rerenderQuery)) {
            nextQuery = nextProps.query;
        }
        if (nextProps.variables !== undefined) {
            nextVariables = nextProps.variables;
        }
        if (nextProps.operationName !== undefined) {
            nextOperationName = nextProps.operationName;
        }
        if (nextProps.responses !== undefined) {
            nextResponses = nextProps.responses;
        }
        if (nextSchema !== this.state.schema ||
            nextQuery !== this.state.query ||
            nextOperationName !== this.state.operationName) {
            this.updateQueryFacts(nextQuery);
            if (this.props.permission) {
                this.reflectQueryVariablesToUI(nextQuery);
            }
        }
        this.setState({
            schema: nextSchema,
            query: nextQuery,
            variables: nextVariables,
            operationName: nextOperationName,
            responses: nextResponses,
        });
    };
    GraphQLEditor.prototype.componentDidUpdate = function () {
        // If this update caused DOM nodes to have changed sizes, update the
        // corresponding CodeMirror instance sizes to match.
        var components = [
            this.queryEditorComponent,
            this.variableEditorComponent,
        ];
        this.codeMirrorSizer.updateSizes(components);
        if (this.resultComponent && Boolean(this.state.subscription)) {
            this.resultComponent.scrollTop = this.resultComponent.scrollHeight;
        }
    };
    // When the component is about to unmount, store any persistable state, such
    // that when the component is remounted, it will use the last used values.
    GraphQLEditor.prototype.componentWillUnmount = function () {
        this._storageSet('query', this.state.query);
        this._storageSet('variables', this.state.variables);
        this._storageSet('operationName', this.state.operationName);
        this._storageSet('editorFlex', this.state.editorFlex);
        this._storageSet('variableEditorHeight', this.state.variableEditorHeight);
    };
    GraphQLEditor.prototype.render = function () {
        var _this = this;
        var children = React.Children.toArray(this.props.children);
        var footer = find_1.default(children, function (child) { return child.type === GraphQLEditor.Footer; });
        var queryWrapStyle = {
            WebkitFlex: this.state.editorFlex,
            flex: this.state.editorFlex,
        };
        var variableOpen = this.state.variableEditorOpen;
        var variableStyle = {
            height: variableOpen ? this.state.variableEditorHeight : null,
        };
        var subscriptionResponse = this.state.responses.length > 1;
        var Tether = this.props.tether;
        return (<div className={cn('graphiql-container', { isActive: this.props.isActive })}>
        <style jsx={true}>{"\n          .graphiql-container {\n            font-family: Open Sans, sans-serif;\n          }\n\n          .docs-button,\n          .schema-button {\n            @p: .absolute, .white, .bgGreen, .pa6, .br2, .z2, .ttu, .fw6, .f14,\n              .ph10, .pointer;\n            padding-bottom: 8px;\n            transform: rotate(-90deg);\n            left: -44px;\n            top: 195px;\n          }\n\n          div.schema-button {\n            @p: .bgLightOrange;\n            left: -53px;\n            top: 120px;\n          }\n\n          .queryWrap {\n            @p: .relative;\n            border-top: 8px solid $darkBlue;\n          }\n\n          .graphiql-button {\n            @p: .white50, .bgDarkBlue, .ttu, .f14, .fw6, .br2, .pointer;\n            padding: 5px 9px 6px 9px;\n            letter-spacing: 0.53px;\n          }\n          .graphiql-button.generate-code {\n            @p: .absolute;\n            top: -57px;\n            right: 25px;\n            z-index: 2;\n          }\n\n          .download-button {\n            @p: .white50, .bgDarkBlue, .ttu, .f14, .fw6, .br2, .pointer,\n              .absolute;\n            right: 25px;\n            padding: 5px 9px 6px 9px;\n            letter-spacing: 0.53px;\n            z-index: 2;\n            background-color: $darkerBlue !important;\n            top: initial !important;\n            bottom: 21px !important;\n          }\n\n          .intro {\n            @p: .absolute, .tlCenter, .top50, .left50, .white20, .f16, .tc;\n            font-family: 'Source Code Pro', 'Consolas', 'Inconsolata',\n              'Droid Sans Mono', 'Monaco', monospace;\n            letter-spacing: 0.6px;\n            width: 235px;\n          }\n          .result-window {\n            @p: .bgDarkBlue, .nosb;\n          }\n\n          .result-window.disableResize :global(.CodeMirror-gutters) {\n            cursor: default !important;\n          }\n\n          .subscription-time {\n            @p: .relative;\n            height: 17px;\n            margin-top: 12px;\n            margin-bottom: 4px;\n            &:before {\n              @p: .absolute, .w100;\n              content: '';\n              top: 9px;\n              left: 95px;\n              border-top: 1px solid $white20;\n            }\n          }\n\n          .subscription-time-text {\n            @p: .bgDarkBlue, .white50, .f12;\n            padding-left: 15px;\n          }\n\n          .listening {\n            @p: .f16, .white40, .absolute, .bottom0;\n            font-family: 'Source Code Pro', 'Consolas', 'Inconsolata',\n              'Droid Sans Mono', 'Monaco', monospace;\n            letter-spacing: 0.6px;\n            padding-left: 24px;\n            padding-bottom: 30px;\n          }\n\n          .onboarding-hint {\n            @p: .absolute, .br2, .z999;\n          }\n          .onboarding-hint.step1 {\n            top: 207px;\n            left: 90px;\n          }\n          .onboarding-hint.step2 {\n            top: 207px;\n            left: 90px;\n          }\n        "}</style>
        <style jsx={true} global={true}>{"\n          .query-header-enter {\n            opacity: 0.01;\n          }\n\n          .query-header-enter.query-header-enter-active {\n            opacity: 1;\n            transition: opacity 500ms ease-in;\n          }\n\n          .query-header-leave {\n            opacity: 1;\n          }\n\n          .query-header-leave.query-header-leave-active {\n            opacity: 0.01;\n            transition: opacity 300ms ease-in;\n          }\n        "}</style>
        <div className="editorWrap">
          <div ref={this.setEditorBarComponent} className="editorBar" onMouseDown={this.handleResizeStart}>
            <div className="queryWrap" style={queryWrapStyle}>
              {this.props.disableAnimation
            ? <QueryHeader_1.default onPrettify={this.handlePrettifyQuery} showEndpoints={this.props.showEndpoints} showQueryTitle={this.props.showQueryTitle}/>
            : <CSSTransitionGroup transitionName="query-header" transitionEnterTimeout={500} transitionLeaveTimeout={300}>
                    {!this.props.disableQueryHeader &&
                <QueryHeader_1.default onPrettify={this.handlePrettifyQuery} showEndpoints={this.props.showEndpoints} showQueryTitle={this.props.showQueryTitle}/>}
                  </CSSTransitionGroup>}
              <QueryEditor_1.QueryEditor ref={this.setQueryEditorComponent} schema={this.state.schema} value={this.state.query} onEdit={this.handleEditQuery} onHintInformationRender={this.handleHintInformationRender} onRunQuery={this.handleEditorRunQuery} disableAutofocus={this.props.disableAutofocus} hideLineNumbers={this.props.hideLineNumbers} hideGutters={this.props.hideGutters} readOnly={this.props.readonly} useVim={this.props.useVim}/>
              <div className="variable-editor" style={variableStyle}>
                <HttpHeaders_1.default headers={this.props.headers} onChange={this.props.onChangeHeaders}/>
                {this.props.showCodeGeneration &&
            <div className="graphiql-button generate-code" onClick={this.props.onClickCodeGeneration}>
                    Generate Code
                  </div>}
                <div className="variable-editor-title" style={{ cursor: variableOpen ? 'row-resize' : 'n-resize' }} onMouseDown={this.handleVariableResizeStart}>
                  {'Query Variables'}
                </div>
                <VariableEditor_1.VariableEditor ref={this.setVariableEditorComponent} value={this.state.variables} variableToType={this.state.variableToType} onEdit={this.handleEditVariables} onHintInformationRender={this.handleHintInformationRender} onRunQuery={this.handleEditorRunQuery}/>
              </div>
              {[
            'STEP3_UNCOMMENT_DESCRIPTION',
            'STEP3_ENTER_MUTATION1_VALUES',
            'STEP3_ENTER_MUTATION2_VALUE',
        ].indexOf(this.props.onboardingStep || '') > -1 &&
            <Tether steps={[
                {
                    step: 'STEP3_UNCOMMENT_DESCRIPTION',
                    title: 'Uncomment the description',
                    description: 'To add the description to the query, just remove the #',
                },
                {
                    step: 'STEP3_ENTER_MUTATION1_VALUES',
                    title: 'This is a mutation',
                    description: 'Enter data for the imageUrl and the description',
                    buttonText: 'Autofill Data',
                },
                {
                    step: 'STEP3_ENTER_MUTATION2_VALUE',
                    title: 'Lets add some more data',
                    description: 'Enter data for the imageUrl and the description',
                    buttonText: 'Autofill Data',
                },
            ]} onClick={this.tetherClick}>
                  <div className={cn('onboarding-hint', {
                step1: this.props.onboardingStep ===
                    'STEP3_UNCOMMENT_DESCRIPTION',
                step2: this.props.onboardingStep ===
                    'STEP3_ENTER_MUTATION1_VALUES' ||
                    this.props.onboardingStep ===
                        'STEP3_ENTER_MUTATION2_VALUE',
            })}/>
                </Tether>}
            </div>
            {this.props.permission &&
            this.props.serviceInformation &&
            <PermissionVariables_1.default variables={this.getVariables()} selectedVariableNames={this.state.selectedVariableNames} onToggleVariableSelection={this.toggleVariableSelection}/>}
            {!this.props.queryOnly &&
            <div className="resultWrap">
                {this.props.isGraphcoolUrl &&
                this.props.showSelectUser &&
                this.props.showViewAs &&
                !this.props.permission &&
                <ResultHeader_1.default showViewAs={this.props.showViewAs} showSelectUser={this.props.showSelectUser} selectedViewer={this.props.selectedViewer} onChangeViewer={this.props.onChangeViewer} showResponseTitle={this.props.showResponseTitle}/>}
                {this.props.tether
                ? <Tether offsetX={18} offsetY={25} steps={[
                    {
                        step: 'STEP3_RUN_QUERY1',
                        title: 'Execute your first query',
                        description: 'You just wrote your first GraphQL Query! Click here to execute it.',
                    },
                    {
                        step: 'STEP3_RUN_MUTATION1',
                        title: 'Run your first mutation',
                        description: 'Awesome! You just wrote your first GraphQL Mutation. Click here to execute it',
                    },
                    {
                        step: 'STEP3_RUN_MUTATION2',
                        title: 'Lets add more data',
                    },
                    {
                        step: 'STEP3_RUN_QUERY2',
                        title: "Let's see how the data changed",
                        description: 'After adding data with the mutations, lets' +
                            ' see how the result of the query changes',
                    },
                ]}>
                      <ExecuteButton_1.default isRunning={Boolean(this.state.subscription)} onRun={this.handleRunQuery} onStop={this.handleStopQuery} operations={this.state.operations}/>
                    </Tether>
                : <ExecuteButton_1.default isRunning={Boolean(this.state.subscription)} onRun={this.handleRunQuery} onStop={this.handleStopQuery} operations={this.state.operations}/>}
                {this.state.isWaitingForResponse && <Spinner_1.default />}
                <div className={'result-window' +
                (this.props.disableResize ? ' disableResize' : '')} ref={this.setResultComponent}>
                  {this.state.responses
                .filter(function (res) { return res && res.date; })
                .map(function (response) {
                return <div key={response.resultID}>
                        {subscriptionResponse &&
                    response.time &&
                    <div className="subscription-time">
                            <div className="subscription-time-text">
                              {ageOfDate_1.default(response.time)}
                            </div>
                          </div>}
                        <ResultViewer_1.ResultViewer value={response.date} hideGutters={_this.props.hideGutters}/>
                      </div>;
            })}
                </div>
                {footer}
                {!this.state.responses ||
                (this.state.responses.length === 0 &&
                    <div className="intro">
                      Hit the Play Button to get a response here
                    </div>)}
                {Boolean(this.state.subscription) &&
                <div className="listening">Listening &hellip;</div>}
                {this.state.responses &&
                this.state.responses.length > 0 &&
                this.props.showDownloadJsonButton &&
                <div className="download-button" onClick={this.handleDownloadJSON}>
                    Download JSON
                  </div>}
              </div>}
          </div>
        </div>
      </div>);
    };
    /**
     * Inspect the query, automatically filling in selection sets for non-leaf
     * fields which do not yet have them.
     *
     * @public
     */
    GraphQLEditor.prototype.autoCompleteLeafs = function () {
        var _a = fillLeafs_1.fillLeafs(this.state.schema, this.state.query, this.props.getDefaultFieldNames), insertions = _a.insertions, result = _a.result;
        if (insertions && insertions.length > 0) {
            var editor_1 = this.queryEditorComponent.getCodeMirror();
            editor_1.operation(function () {
                var cursor = editor_1.getCursor();
                var cursorIndex = editor_1.indexFromPos(cursor);
                editor_1.setValue(result);
                var added = 0;
                try {
                    var markers_1 = insertions.map(function (_a) {
                        var index = _a.index, str = _a.str;
                        return editor_1.markText(editor_1.posFromIndex(index + added), editor_1.posFromIndex(index + (added += str.length)), {
                            className: 'autoInsertedLeaf',
                            clearOnEnter: true,
                            title: 'Automatically added leaf fields',
                        });
                    });
                    setTimeout(function () { return markers_1.forEach(function (marker) { return marker.clear(); }); }, 7000);
                }
                catch (e) {
                    //
                }
                var newCursorIndex = cursorIndex;
                insertions.forEach(function (_a) {
                    var index = _a.index, str = _a.str;
                    if (index < cursorIndex && str) {
                        newCursorIndex += str.length;
                    }
                });
                editor_1.setCursor(editor_1.posFromIndex(newCursorIndex));
            });
        }
        return result;
    };
    // Private methods
    GraphQLEditor.prototype._ensureOfSchema = function () {
        var _this = this;
        // Only perform introspection if a schema is not provided (undefined)
        if (this.state.schema !== undefined) {
            return;
        }
        var fetcher = this.props.fetcher;
        var fetch = observableToPromise(fetcher({ query: introspectionQueries_1.introspectionQuery }));
        if (!isPromise(fetch)) {
            this.setState({
                responses: [
                    {
                        date: 'Fetcher did not return a Promise for introspection.',
                        time: new Date(),
                    },
                ],
            });
            return;
        }
        fetch
            .then(function (result) {
            if (result.data) {
                return result;
            }
            // Try the stock introspection query first, falling back on the
            // sans-subscriptions query for services which do not yet support it.
            var fetch2 = observableToPromise(fetcher({
                query: introspectionQueries_1.introspectionQuerySansSubscriptions,
            }));
            if (!isPromise(fetch)) {
                throw new Error('Fetcher did not return a Promise for introspection.');
            }
            return fetch2;
        })
            .then(function (result) {
            // If a schema was provided while this fetch was underway, then
            // satisfy the race condition by respecting the already
            // provided schema.
            if (_this.state.schema !== undefined) {
                return;
            }
            if (result && result.data) {
                var schema = graphql_1.buildClientSchema(result.data);
                var queryFacts = getQueryFacts_1.default(schema, _this.state.query);
                _this.setState(__assign({ schema: schema }, queryFacts));
            }
            else {
                var responseString = typeof result === 'string'
                    ? result
                    : JSON.stringify(result, null, 2);
                _this.setState({
                    // Set schema to `null` to explicitly indicate that no schema exists.
                    schema: null,
                    responses: [{ date: responseString, time: new Date() }],
                });
            }
        })
            .catch(function (error) {
            _this.setState({
                schema: null,
                responses: [
                    { date: error && String(error.stack || error), time: new Date() },
                ],
            });
        });
    };
    GraphQLEditor.prototype._fetchQuery = function (query, variables, operationName, cb) {
        var _this = this;
        var fetcher = this.props.fetcher;
        var jsonVariables = null;
        try {
            jsonVariables =
                variables && variables.trim() !== '' ? JSON.parse(variables) : null;
        }
        catch (error) {
            throw new Error("Variables are invalid JSON: " + error.message + ".");
        }
        if (typeof jsonVariables !== 'object') {
            throw new Error('Variables are not a JSON object.');
        }
        var fetch = fetcher({
            query: query,
            variables: jsonVariables,
            operationName: operationName,
        });
        if (isPromise(fetch)) {
            // If fetcher returned a Promise, then call the callback when the promise
            // resolves, otherwise handle the error.
            fetch.then(cb).catch(function (error) {
                _this.setState({
                    isWaitingForResponse: false,
                    responses: [
                        { date: error && String(error.stack || error), time: new Date() },
                    ],
                });
            });
        }
        else if (isObservable(fetch)) {
            // If the fetcher returned an Observable, then subscribe to it, calling
            // the callback on each next value, and handling both errors and the
            // completion of the Observable. Returns a Subscription object.
            var subscription = fetch.subscribe({
                // next: cb,
                next: cb,
                error: function (error) {
                    _this.setState({
                        isWaitingForResponse: false,
                        responses: [
                            {
                                date: error && String(error.stack || error),
                                time: new Date(),
                            },
                        ],
                        subscription: null,
                    });
                },
                complete: function () {
                    _this.setState({
                        isWaitingForResponse: false,
                        subscription: null,
                    });
                },
            });
            return subscription;
        }
        else {
            throw new Error('Fetcher did not return Promise or Observable.');
        }
    };
    GraphQLEditor.prototype._runQueryAtCursor = function () {
        if (this.state.subscription) {
            this.handleStopQuery();
            return;
        }
        var operationName;
        var operations = this.state.operations;
        if (operations) {
            var editor = this.queryEditorComponent.getCodeMirror();
            if (editor.hasFocus()) {
                var cursor = editor.getCursor();
                var cursorIndex = editor.indexFromPos(cursor);
                // Loop through all operations to see if one contains the cursor.
                for (var _i = 0, operations_1 = operations; _i < operations_1.length; _i++) {
                    var operation = operations_1[_i];
                    if (operation.loc.start <= cursorIndex &&
                        operation.loc.end >= cursorIndex) {
                        operationName = operation.name && operation.name.value;
                        break;
                    }
                }
            }
        }
        this.handleRunQuery(operationName);
    };
    GraphQLEditor.prototype._didClickDragBar = function (event) {
        // Only for primary unmodified clicks
        if (event.button !== 0 || event.ctrlKey) {
            return false;
        }
        var target = event.target;
        // We use codemirror's gutter as the drag bar.
        if (target.className.indexOf &&
            target.className.indexOf('CodeMirror-gutter') !== 0) {
            return false;
        }
        // Specifically the result window's drag bar.
        var resultWindow = ReactDOM.findDOMNode(this.resultComponent);
        while (target) {
            if (target === resultWindow) {
                return true;
            }
            target = target.parentNode;
        }
        return false;
    };
    GraphQLEditor.prototype.getSelectedVariables = function () {
        var selectedVariableNames = this.state.selectedVariableNames;
        var variables = this.getVariables();
        return lodash_1.flatMap(Object.keys(variables), function (group) { return variables[group]; }).filter(function (variable) { return selectedVariableNames.includes(variable.name); });
    };
    GraphQLEditor.prototype.getPermissionQueryArguments = function () {
        var permission = this.props.permission;
        var serviceInformation = this.props.serviceInformation;
        if (permission.modelName && permission.modelName.length > 0) {
            var model = serviceInformation.models.find(function (m) { return m.name === permission.modelName; });
            if (model) {
                return model.update;
            }
        }
        if (permission.relationName && permission.relationName.length > 0) {
            var relation = serviceInformation.relations.find(function (r) { return r.name === permission.relationName; }).permissionQueryArguments;
            if (relation) {
                return relation;
            }
        }
        return [];
    };
    GraphQLEditor.prototype.getVariables = function () {
        var args = this.getPermissionQueryArguments();
        var variables = lodash_1.groupBy(args, function (arg) { return arg.group; });
        return variables;
    };
    return GraphQLEditor;
}(React.PureComponent));
exports.GraphQLEditor = GraphQLEditor;
// Configure the UI by providing this Component as a child of GraphQLEditor.
GraphQLEditor.Logo = function GraphiQLLogo(props) {
    return (<div className="title">
      {props.children ||
        <span>
          {'Graph'}
          <em>
            {'i'}
          </em>
          {'QL'}
        </span>}
    </div>);
};
// Configure the UI by providing this Component as a child of GraphQLEditor.
GraphQLEditor.Toolbar = function GraphiQLToolbar(props) {
    return (<div className="toolbar">
      {props.children}
    </div>);
};
// Add a button to the Toolbar.
GraphQLEditor.ToolbarButton = ToolbarButton_1.ToolbarButton;
// Configure the UI by providing this Component as a child of GraphQLEditor.
GraphQLEditor.Footer = function GraphiQLFooter(props) {
    return (<div className="footer">
      {props.children}
    </div>);
};
// Duck-type promise detection.
function isPromise(value) {
    return typeof value === 'object' && typeof value.then === 'function';
}
// Duck-type Observable.take(1).toPromise()
function observableToPromise(observable) {
    if (!isObservable(observable)) {
        return observable;
    }
    return new Promise(function (resolve, reject) {
        var subscription = observable.subscribe(function (v) {
            resolve(v);
            subscription.unsubscribe();
        }, reject, function () {
            reject(new Error('no value resolved'));
        });
    });
}
// Duck-type observable detection.
function isObservable(value) {
    return typeof value === 'object' && typeof value.subscribe === 'function';
}
//# sourceMappingURL=GraphQLEditor.jsx.map