"use strict";

Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var validation_1 = require("graphql/validation");
function putVariablesToQuery(query, variables) {
    var newQuery = query;
    try {
        var ast = graphql_1.parse(query);
        // let nameEnd = -1
        var selectionStart_1 = -1;
        graphql_1.visit(ast, {
            // Name(node) {
            //   if (nameEnd === -1) {
            //     nameEnd = node.loc.end
            //     debugger
            //   }
            // },
            SelectionSet: function SelectionSet(node) {
                if (selectionStart_1 === -1) {
                    selectionStart_1 = node.loc.start;
                }
            }
        });
        // newQuery = query.slice(0, nameEnd) + renderVariables(variables) + query.slice(selectionStart, query.length)
        newQuery = 'query ' + renderVariables(variables) + query.slice(selectionStart_1, query.length);
    } catch (e) {
        //
    }
    return newQuery;
}
exports.putVariablesToQuery = putVariablesToQuery;
function extractSelection(query) {
    var newQuery = query;
    try {
        var ast = graphql_1.parse(query);
        var selectionStart_2 = -1;
        var selectionEnd_1 = -1;
        graphql_1.visit(ast, {
            SelectionSet: function SelectionSet(node) {
                // take first selection
                if (selectionStart_2 === -1 && selectionEnd_1 === -1) {
                    selectionStart_2 = node.loc.start;
                    selectionEnd_1 = node.loc.end;
                }
            }
        });
        newQuery = query.slice(selectionStart_2, selectionEnd_1);
    } catch (e) {
        //
    }
    return newQuery;
}
exports.extractSelection = extractSelection;
function addVarsAndName(modelNamePlural, query, vars, schema) {
    var newQuery = query;
    try {
        var ast = graphql_1.parse(query);
        var selectionStart_3 = -1;
        var selectionEnd_2 = -1;
        graphql_1.visit(ast, {
            SelectionSet: function SelectionSet(node) {
                // take first selection
                if (selectionStart_3 === -1 && selectionEnd_2 === -1) {
                    selectionStart_3 = node.loc.start;
                    selectionEnd_2 = node.loc.end;
                }
            }
        });
        var variables = getVariableNamesFromQuery(query, false, schema).variables;
        var mappedVariables = variables.map(function (variable) {
            return vars.find(function (arg) {
                return arg.name === variable;
            });
        });
        var printedVariables = renderVariables(mappedVariables);
        newQuery = "query " + printedVariables + query.slice(selectionStart_3, query.length);
    } catch (e) {
        //
    }
    return newQuery;
}
exports.addVarsAndName = addVarsAndName;
function getVariableNamesFromQuery(query, definitionOnly, schema) {
    if (definitionOnly === void 0) {
        definitionOnly = false;
    }
    var variables = new Set();
    var valid = true;
    try {
        var ast = graphql_1.parse(query);
        var config = {};
        if (definitionOnly) {
            config = {
                VariableDefinition: function VariableDefinition(node) {
                    variables.add('$' + node.variable.name.value);
                }
            };
        } else {
            config = {
                Variable: function Variable(node) {
                    variables.add('$' + node.name.value);
                }
            };
        }
        graphql_1.visit(ast, config);
        var validation = validation_1.validate(schema, ast);
        if (validation.length > 0) {
            valid = false;
        }
    } catch (e) {
        valid = false;
    }
    var result = Array.from(variables);
    return {
        variables: result,
        valid: valid
    };
}
exports.getVariableNamesFromQuery = getVariableNamesFromQuery;
function renderVariables(variables) {
    if (variables.length === 0) {
        return ' ';
    }
    return '(' + variables.map(function (variable) {
        return variable.name + ": " + renderType(variable);
    }).join(', ') + ') ';
}
function renderType(variable) {
    return variable.typeName + '!';
}
exports.renderType = renderType;
//# sourceMappingURL=ast.js.map