"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var fetch = require("isomorphic-fetch");
var constants_1 = require("../constants");
var Modal = require("react-modal");
var graphcool_styles_1 = require("graphcool-styles");
var Table_1 = require("./SelectUserPopup/Table");
var SearchBox_1 = require("./Playground/DocExplorer/SearchBox");
var Immutable = require("seamless-immutable");
var SelectUserPopup = /** @class */ (function (_super) {
    __extends(SelectUserPopup, _super);
    function SelectUserPopup(props) {
        var _this = _super.call(this, props) || this;
        _this.changeUserModelName = function (e) {
            _this.props.onChangeUserModelName(e.target.value);
        };
        _this.handleRowSelection = function (_a) {
            var index = _a.index, rowData = _a.rowData;
            if (index === _this.state.selectedRowIndex) {
                return;
            }
            _this.setState(function (state) {
                var users = state.users;
                if (state.selectedRowIndex > -1) {
                    users = Immutable.setIn(users, [state.selectedRowIndex, 'selected'], false);
                }
                users = Immutable.setIn(users, [index, 'selected'], true);
                return __assign({}, state, { users: users, selectedRowIndex: index });
            });
            _this.props.onSelectUser(rowData);
        };
        _this.handleSearch = function (value) {
            _this.setState({ query: value }, function () {
                var _a = _this.state, startIndex = _a.startIndex, stopIndex = _a.stopIndex;
                _this.getUsers({ startIndex: startIndex, stopIndex: stopIndex });
            });
        };
        _this.getUsers = function (_a, userFieldsInput) {
            var startIndex = _a.startIndex, stopIndex = _a.stopIndex;
            var query = _this.state.query;
            var userFields = userFieldsInput || _this.props.userFields;
            if (userFields.length === 0) {
                return;
            }
            var filter = '';
            if (query && query.length > 0) {
                filter = ' filter: { OR: [';
                var whiteList_1 = ['ID', 'String', 'Enum'];
                var filtered = userFields.filter(function (field) {
                    var typeName = field.type.name || field.type.ofType.name;
                    return whiteList_1.indexOf(typeName) > -1;
                });
                filter += filtered
                    .map(function (field) { return "{" + field.name + "_contains: \"" + query + "\"}"; })
                    .join(',\n');
                filter += ']}';
            }
            var count = stopIndex - startIndex;
            var userQuery = "\n      {\n        _all" + _this.props.userModelName + "sMeta {\n          count\n        }\n        all" + _this.props
                .userModelName + "s(skip: " + startIndex + " first: " + count + filter + "){\n          " + userFields.map(function (f) { return f.name; }).join('\n') + "\n        }\n      }\n    ";
            fetch(_this.props.endpointUrl, {
                // tslint:disable-line
                method: 'post',
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: "Bearer " + _this.props.adminAuthToken,
                    'X-GraphCool-Source': 'playground',
                },
                body: JSON.stringify({ query: userQuery }),
            })
                .then(function (res) { return res.json(); })
                .then(function (res) {
                if (!res.data) {
                    return;
                }
                var allUsers = res.data["all" + _this.props.userModelName + "s"];
                /* tslint:disable */
                var _allUsersMeta = res.data["_all" + _this.props.userModelName + "sMeta"];
                var users = _this.state.users;
                // reset data if search changed
                if (query !== _this.lastQuery) {
                    users = Immutable([]);
                }
                allUsers.forEach(function (user, i) {
                    users = Immutable.set(users, i + startIndex, user);
                });
                var newState = {
                    users: users,
                    count: _allUsersMeta.count,
                };
                if (_this.lastQuery !== _this.state.query) {
                    newState.scrollToIndex = 0;
                    setTimeout(function () {
                        _this.setState({
                            scrollToIndex: undefined,
                        });
                    }, 150);
                }
                _this.setState(newState);
                _this.lastQuery = query;
            })
                .catch(function (e) { return console.error(e); });
        };
        _this.state = {
            startIndex: 0,
            stopIndex: 50,
            users: Immutable([]),
            query: '',
            count: 0,
            selectedRowIndex: -1,
            scrollToIndex: undefined,
        };
        _this.getUsers({ startIndex: 0, stopIndex: 50 }, props.userFields);
        _this.style = __assign({}, constants_1.modalStyle, { overlay: constants_1.modalStyle.overlay, content: __assign({}, constants_1.modalStyle.content, { width: 'auto', minWidth: '600px', maxWidth: window.innerWidth - 100 + 'px' }) });
        global.s = _this;
        return _this;
    }
    SelectUserPopup.prototype.componentWillReceiveProps = function (nextProps) {
        var _a = this.state, startIndex = _a.startIndex, stopIndex = _a.stopIndex;
        if (nextProps.userFields.length !== this.props.userFields.length) {
            this.getUsers({ startIndex: startIndex, stopIndex: stopIndex }, nextProps.userFields);
        }
    };
    SelectUserPopup.prototype.componentDidUpdate = function (props) {
        var _a = this.state, startIndex = _a.startIndex, stopIndex = _a.stopIndex;
        if (props.userModelName !== this.props.userModelName) {
            this.getUsers({ startIndex: startIndex, stopIndex: stopIndex }, this.props.userFields);
        }
    };
    SelectUserPopup.prototype.render = function () {
        // put id to beginning
        return (<Modal isOpen={this.props.isOpen} onRequestClose={this.props.onRequestClose} contentLabel="Select a User" style={this.style}>
        <style jsx={true}>{"\n          .select-user-popup {\n            @inherit: .bgWhite, .relative, .mh25;\n          }\n          .title-wrapper {\n            @inherit: .flex, .w100, .itemsCenter, .justifyCenter, .bb, .bBlack10;\n            padding: 45px;\n          }\n          .title {\n            @inherit: .fw3, .f38;\n            letter-spacing: 0.54px;\n          }\n          .search {\n            @inherit: .absolute, .w100, .bbox, .ph38, .z2, .flex, .justifyCenter;\n            margin-top: -24px;\n          }\n          .search-box {\n            flex: 0 1 400px;\n          }\n          .selected-user {\n            @inherit: .ml25, .flex, .flexColumn, .justifyCenter, .itemsCenter;\n          }\n          .selected-user-id {\n            @inherit: .bgBlack04, .pa6, .br2, .black60, .f14, .fw3, .mt10;\n            font-family: 'Source Code Pro', 'Consolas', 'Inconsolata',\n              'Droid Sans Mono', 'Monaco', monospace;\n          }\n          select {\n            @p: .mt16, .f20;\n          }\n          .select-user {\n            @p: .absolute;\n            top: 25px;\n            left: 38px;\n          }\n        "}</style>
        <style jsx={true} global={true}>{"\n          .popup-x {\n            @inherit: .absolute, .right0, .top0, .pointer, .pt25, .pr25;\n          }\n        "}</style>
        <div className="select-user-popup">
          <div className="select-user">
            <div>Select the User Model</div>
            <select value={this.props.userModelName} onChange={this.changeUserModelName}>
              {this.props.modelNames.map(function (name) {
            return <option value={name}>
                  {name}
                </option>;
        })}
            </select>
          </div>
          <div className="title-wrapper">
            <div className="title">
              Select a {this.props.userModelName}
            </div>
            {this.state.selectedRowIndex > -1 &&
            <div className="selected-user">
                <div>
                  Selected {this.props.userModelName} ID
                </div>
                <div className="selected-user-id">
                  {this.state.users[this.state.selectedRowIndex].id}
                </div>
              </div>}
          </div>
          <graphcool_styles_1.Icon src={require('graphcool-styles/icons/stroke/cross.svg')} stroke={true} width={25} height={25} strokeWidth={2} className="popup-x" color={graphcool_styles_1.$v.gray50} onClick={this.props.onRequestClose}/>
          <div className="search">
            <div className="search-box">
              <SearchBox_1.default placeholder="Search for a user ..." onSearch={this.handleSearch} isShown={true} clean={true}/>
            </div>
          </div>
          <Table_1.default fields={this.props.userFields} rows={this.state.users} rowCount={this.state.count} loadMoreRows={this.getUsers} onRowSelection={this.handleRowSelection} scrollToIndex={this.state.scrollToIndex}/>
        </div>
      </Modal>);
    };
    return SelectUserPopup;
}(React.Component));
exports.default = SelectUserPopup;
//# sourceMappingURL=SelectUserPopup.jsx.map